@@ -60,7 +60,8 @@ class Akonadi::CollectionFetchJobPrivate : public JobPrivate
 
       mEmitTimer->stop(); // in case we are called by result()
       if ( !mPendingCollections.isEmpty() ) {
-        emit q->collectionsReceived( mPendingCollections );
+        if ( !q->error() )
+          emit q->collectionsReceived( mPendingCollections );
         mPendingCollections.clear();
       }
     }
@@ -77,8 +78,8 @@ CollectionFetchJob::CollectionFetchJob( const Collection &collection, Type type,
   d->mEmitTimer = new QTimer( this );
   d->mEmitTimer->setSingleShot( true );
   d->mEmitTimer->setInterval( 100 );
-  connect( d->mEmitTimer, SIGNAL(timeout()), this, SLOT(timeout()) );
-  connect( this, SIGNAL(result(KJob*)), this, SLOT(timeout()) );
+  connect( d->mEmitTimer, SIGNAL( timeout() ), this, SLOT( timeout() ) );
+  connect( this, SIGNAL( result( KJob* ) ), this, SLOT( timeout() ) );
 }
 
 CollectionFetchJob::CollectionFetchJob( const Collection::List & cols, QObject * parent )
@@ -97,8 +98,8 @@ CollectionFetchJob::CollectionFetchJob( const Collection::List & cols, QObject *
   d->mEmitTimer = new QTimer( this );
   d->mEmitTimer->setSingleShot( true );
   d->mEmitTimer->setInterval( 100 );
-  connect( d->mEmitTimer, SIGNAL(timeout()), this, SLOT(timeout()) );
-  connect( this, SIGNAL(result(KJob*)), this, SLOT(timeout()) );
+  connect( d->mEmitTimer, SIGNAL( timeout() ), this, SLOT( timeout() ) );
+  connect( this, SIGNAL( result( KJob* ) ), this, SLOT( timeout() ) );
 }
 
 CollectionFetchJob::~CollectionFetchJob()
@@ -138,7 +139,7 @@ void CollectionFetchJob::doStart()
     else
       command += " " AKONADI_CMD_RID;
   }
-  if ( d->mScope.includeUnubscribed() )
+  if ( d->mScope.includeUnsubscribed() )
     command += " LIST ";
   else
     command += " LSUB ";
@@ -168,6 +169,7 @@ void CollectionFetchJob::doStart()
   QList<QByteArray> filter;
   if ( !d->mScope.resource().isEmpty() ) {
     filter.append( "RESOURCE" );
+    // FIXME: Does this need to be quoted??
     filter.append( d->mScope.resource().toUtf8() );
   }
 
@@ -175,6 +177,7 @@ void CollectionFetchJob::doStart()
     filter.append( "MIMETYPE" );
     QList<QByteArray> mts;
     foreach ( const QString &mt, d->mScope.contentMimeTypes() )
+      // FIXME: Does this need to be quoted??
       mts.append( mt.toUtf8() );
     filter.append( '(' + ImapParser::join( mts, " " ) + ')' );
   }
