@@ -69,14 +69,14 @@ class AKONADI_EXPORT CollectionFilterProxyModel : public QSortFilterProxyModel
     virtual ~CollectionFilterProxyModel();
 
     /**
-     * Add mime types to be shown by the filter.
+     * Adds a list of mime types to be shown by the filter.
      *
      * @param mimeTypes A list of mime types to be shown.
      */
     void addMimeTypeFilters( const QStringList &mimeTypes );
 
     /**
-     * Add mime type to be shown by the filter.
+     * Adds a mime type to be shown by the filter.
      *
      * @param mimeType A mime type to be shown.
      */
@@ -88,19 +88,19 @@ class AKONADI_EXPORT CollectionFilterProxyModel : public QSortFilterProxyModel
     QStringList mimeTypeFilters() const;
 
     /**
-     * Clear all mime type filters.
+     * Clears all mime type filters.
      */
     void clearFilters();
 
+    virtual Qt::ItemFlags flags( const QModelIndex& index ) const;
+
   protected:
-    virtual bool filterAcceptsRow( int sourceRow, const QModelIndex &sourceParent) const;
+    virtual bool filterAcceptsRow( int sourceRow, const QModelIndex &sourceParent ) const;
 
   private:
     //@cond PRIVATE
     class Private;
     Private* const d;
-
-    Q_PRIVATE_SLOT( d, void slotReset() )
     //@endcond
 };
 
