@@ -243,10 +243,10 @@ class CollectionSync::Private
       to see if any of them can be processed by now. If not, they are moved to
       the closest ancestor available.
     */
-    void processPendingRemoteNodes( LocalNode *localRoot )
+    void processPendingRemoteNodes( LocalNode *_localRoot )
     {
-      QList<RemoteNode*> pendingRemoteNodes( localRoot->pendingRemoteNodes );
-      localRoot->pendingRemoteNodes.clear();
+      QList<RemoteNode*> pendingRemoteNodes( _localRoot->pendingRemoteNodes );
+      _localRoot->pendingRemoteNodes.clear();
       QHash<LocalNode*, QList<RemoteNode*> > pendingCreations;
       foreach ( RemoteNode *remoteNode, pendingRemoteNodes ) {
         // step 1: see if we have a matching local node already
@@ -286,11 +286,17 @@ class CollectionSync::Private
     */
     void updateLocalCollection( LocalNode *localNode, RemoteNode *remoteNode )
     {
-      ++pendingJobs;
       Collection upd( remoteNode->collection );
       upd.setId( localNode->collection.id() );
-      CollectionModifyJob *mod = new CollectionModifyJob( upd, q );
-      connect( mod, SIGNAL(result(KJob*)), q, SLOT(updateLocalCollectionResult(KJob*)) );
+      {
+        // ### HACK to work around the implicit move attempts of CollectionModifyJob
+        // which we do explicitly below
+        Collection c( upd );
+        c.setParentCollection( localNode->collection.parentCollection() );
+        ++pendingJobs;
+        CollectionModifyJob *mod = new CollectionModifyJob( c, q );
+        connect( mod, SIGNAL( result( KJob* ) ), q, SLOT( updateLocalCollectionResult( KJob* ) ) );
+      }
 
       // detecting moves is only possible with global RIDs
       if ( !hierarchicalRIDs ) {
@@ -301,7 +307,7 @@ class CollectionSync::Private
         if ( newParent && oldParent != newParent ) {
           ++pendingJobs;
           CollectionMoveJob *move = new CollectionMoveJob( upd, newParent->collection, q );
-          connect( move, SIGNAL(result(KJob*)), q, SLOT(updateLocalCollectionResult(KJob*)) );
+          connect( move, SIGNAL( result( KJob* ) ), q, SLOT( updateLocalCollectionResult( KJob* ) ) );
         }
       }
 
@@ -332,7 +338,7 @@ class CollectionSync::Private
         CollectionCreateJob *create = new CollectionCreateJob( col, q );
         create->setProperty( LOCAL_NODE, QVariant::fromValue( localParent ) );
         create->setProperty( REMOTE_NODE, QVariant::fromValue( remoteNode ) );
-        connect( create, SIGNAL(result(KJob*)), q, SLOT(createLocalCollectionResult(KJob*)) );
+        connect( create, SIGNAL( result( KJob* ) ), q, SLOT( createLocalCollectionResult( KJob* ) ) );
       }
     }
 
@@ -415,15 +421,20 @@ class CollectionSync::Private
       foreach ( const Collection &col, cols ) {
         ++pendingJobs;
         CollectionDeleteJob *job = new CollectionDeleteJob( col, q );
-        connect( job, SIGNAL(result(KJob*)), q, SLOT(deleteLocalCollectionsResult(KJob*)) );
+        connect( job, SIGNAL( result( KJob* ) ), q, SLOT( deleteLocalCollectionsResult( KJob* ) ) );
+
+        // It can happen that the groupware servers report us deleted collections
+        // twice, in this case this collection delete job will fail on the second try.
+        // To avoid a rollback of the complete transaction we gracefully allow the job
+        // to fail :)
+        q->setIgnoreJobFailure( job );
       }
     }
 
-    void deleteLocalCollectionsResult( KJob *job )
+    void deleteLocalCollectionsResult( KJob* )
     {
       --pendingJobs;
-      if ( job->error() )
-        return; // handled by the base class
+
       ++progress;
       checkDone();
     }
@@ -562,8 +573,9 @@ void CollectionSync::doStart()
   CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive, this );
   job->fetchScope().setResource( d->resourceId );
   job->fetchScope().setAncestorRetrieval( CollectionFetchScope::Parent );
-  connect( job, SIGNAL(collectionsReceived(Akonadi::Collection::List)), SLOT(localCollectionsReceived(Akonadi::Collection::List)) );
-  connect( job, SIGNAL(result(KJob*)), SLOT(localCollectionFetchResult(KJob*)) );
+  connect( job, SIGNAL( collectionsReceived( const Akonadi::Collection::List& ) ),
+           SLOT( localCollectionsReceived( const Akonadi::Collection::List& ) ) );
+  connect( job, SIGNAL( result( KJob* ) ), SLOT( localCollectionFetchResult( KJob* ) ) );
 }
 
 void CollectionSync::setStreamingEnabled( bool streaming )
