@@ -22,7 +22,6 @@
 #include "ui_controlprogressindicator.h"
 #include "selftestdialog_p.h"
 #include "erroroverlay_p.h"
-#include "firstrun_p.h"
 
 #include <kdebug.h>
 #include <kglobal.h>
@@ -35,6 +34,9 @@
 
 using namespace Akonadi;
 
+namespace Akonadi {
+namespace Internal {
+
 class ControlProgressIndicator : public QFrame
 {
   public:
@@ -64,7 +66,9 @@ class StaticControl : public Control
     StaticControl() : Control() {}
 };
 
-K_GLOBAL_STATIC( StaticControl, s_instance )
+}
+
+K_GLOBAL_STATIC( Internal::StaticControl, s_instance )
 
 /**
  * @internal
@@ -75,13 +79,9 @@ class Control::Private
     Private( Control *parent )
       : mParent( parent ), mEventLoop( 0 ),
         mProgressIndicator( 0 ),
-        mFirstRunner( 0 ),
         mSuccess( false ),
         mStarting( false ), mStopping( false )
     {
-      KGlobal::locale()->insertCatalog( QString::fromLatin1("libakonadi") );
-      if ( ServerManager::isRunning() )
-        mFirstRunner = new Firstrun( mParent );
     }
 
     ~Private()
@@ -92,7 +92,7 @@ class Control::Private
     void setupProgressIndicator( const QString &msg, QWidget *parent = 0 )
     {
       if ( !mProgressIndicator )
-        mProgressIndicator = new ControlProgressIndicator( parent );
+        mProgressIndicator = new Internal::ControlProgressIndicator( parent );
 
       mProgressIndicator->setMessage( msg );
     }
@@ -111,14 +111,12 @@ class Control::Private
     }
 
     bool exec();
-    void serverStarted();
-    void serverStopped();
+    void serverStateChanged(ServerManager::State state);
 
     QPointer<Control> mParent;
     QEventLoop *mEventLoop;
-    QPointer<ControlProgressIndicator> mProgressIndicator;
+    QPointer<Internal::ControlProgressIndicator> mProgressIndicator;
     QList<QPointer<QWidget> > mPendingOverlays;
-    Firstrun *mFirstRunner;
     bool mSuccess;
 
     bool mStarting;
@@ -130,10 +128,8 @@ bool Control::Private::exec()
   if ( mProgressIndicator )
     mProgressIndicator->show();
 
-  kDebug() << "Starting Akonadi (using an event loop).";
+  kDebug() << "Starting/Stopping Akonadi (using an event loop).";
   mEventLoop = new QEventLoop( mParent );
-  // safety timeout
-  QTimer::singleShot( 10000, mEventLoop, SLOT(quit()) );
   mEventLoop->exec();
   mEventLoop->deleteLater();
   mEventLoop = 0;
@@ -159,30 +155,20 @@ bool Control::Private::exec()
   return rv;
 }
 
-void Control::Private::serverStarted()
+void Control::Private::serverStateChanged(ServerManager::State state)
 {
-  if ( mEventLoop && mEventLoop->isRunning() && mStarting ) {
+  kDebug() << state;
+  if ( mEventLoop && mEventLoop->isRunning() ) {
     mEventLoop->quit();
-    mSuccess = true;
-  }
-  if ( !mFirstRunner )
-    mFirstRunner = new Firstrun( mParent );
-}
-
-void Control::Private::serverStopped()
-{
-  if ( mEventLoop && mEventLoop->isRunning() && mStopping ) {
-    mEventLoop->quit();
-    mSuccess = true;
+    mSuccess = (mStarting && state == ServerManager::Running) || (mStopping && state == ServerManager::NotRunning);
   }
 }
 
-
 Control::Control()
   : d( new Private( this ) )
 {
-  connect( ServerManager::self(), SIGNAL( started() ), SLOT( serverStarted() ) );
-  connect( ServerManager::self(), SIGNAL( stopped() ), SLOT( serverStopped() ) );
+  connect( ServerManager::self(), SIGNAL( stateChanged( Akonadi::ServerManager::State ) ),
+           SLOT( serverStateChanged( Akonadi::ServerManager::State ) ) );
   // mProgressIndicator is a widget, so it better be deleted before the QApplication is deleted
   // Otherwise we get a crash in QCursor code with Qt-4.5
   if ( QCoreApplication::instance() )
@@ -196,19 +182,25 @@ Control::~Control()
 
 bool Control::start()
 {
-  if ( s_instance->d->mStopping )
+  if ( ServerManager::state() == ServerManager::Stopping ) {
+    kDebug() << "Server is currently being stopped, wont try to start it now";
     return false;
-  if ( ServerManager::isRunning() || s_instance->d->mEventLoop )
+  }
+  if ( ServerManager::isRunning() || s_instance->d->mEventLoop ) {
+    kDebug() << "Server is already running";
     return true;
+  }
   s_instance->d->mStarting = true;
-  if ( !ServerManager::start() )
+  if ( !ServerManager::start() ) {
+    kDebug() << "ServerManager::start failed -> return false";
     return false;
+  }
   return s_instance->d->exec();
 }
 
 bool Control::stop()
 {
-  if ( s_instance->d->mStarting )
+  if ( ServerManager::state() == ServerManager::Starting )
     return false;
   if ( !ServerManager::isRunning() || s_instance->d->mEventLoop )
     return true;
@@ -253,7 +245,9 @@ void Control::widgetNeedsAkonadi(QWidget * widget)
   s_instance->d->mPendingOverlays.append( widget );
   // delay the overlay creation since we rely on widget being reparented
   // correctly already
-  QTimer::singleShot( 0, s_instance, SLOT(createErrorOverlays()) );
+  QTimer::singleShot( 0, s_instance, SLOT( createErrorOverlays() ) );
+}
+
 }
 
 #include "control.moc"
