@@ -23,10 +23,10 @@
 #include "collection.h"
 #include "entity_p.h"
 #include "imapparser_p.h"
-#include "itemserializer.h"
+#include "itemserializer_p.h"
 #include "job_p.h"
 #include "item_p.h"
-#include "protocolhelper.h"
+#include "protocolhelper_p.h"
 
 #include <kdebug.h>
 
@@ -58,11 +58,16 @@ QByteArray ItemModifyJobPrivate::nextPartHeader()
     int version = 0;
     ItemSerializer::serialize( mItem, label, mPendingData, version );
     command += ' ' + ProtocolHelper::encodePartIdentifier( ProtocolHelper::PartPayload, label, version );
-    command += ".SILENT {" + QByteArray::number( mPendingData.size() ) + '}';
-    if ( mPendingData.size() > 0 )
-      command += '\n';
-    else
+    command += ".SILENT";
+    if ( mPendingData.size() > 0 ) {
+      command += " {" + QByteArray::number( mPendingData.size() ) + "}\n";
+    } else {
+      if ( mPendingData.isNull() )
+        command += " NIL";
+      else
+        command += " \"\"";
       command += nextPartHeader();
+    }
   } else {
     command += ")\n";
   }
@@ -76,6 +81,7 @@ ItemModifyJob::ItemModifyJob( const Item &item, QObject * parent )
   Q_D( ItemModifyJob );
 
   d->mOperations.insert( ItemModifyJobPrivate::RemoteId );
+  d->mOperations.insert( ItemModifyJobPrivate::RemoteRevision );
 }
 
 ItemModifyJob::~ItemModifyJob()
@@ -92,7 +98,13 @@ void ItemModifyJob::doStart()
       case ItemModifyJobPrivate::RemoteId:
         if ( !d->mItem.remoteId().isNull() ) {
           changes << "REMOTEID.SILENT";
-          changes << ImapParser::quote( d->mItem.remoteId().toLatin1() );
+          changes << ImapParser::quote( d->mItem.remoteId().toUtf8() );
+        }
+        break;
+      case ItemModifyJobPrivate::RemoteRevision:
+        if ( !d->mItem.remoteRevision().isNull() ) {
+          changes << "REMOTEREVISION.SILENT";
+          changes << ImapParser::quote( d->mItem.remoteRevision().toUtf8() );
         }
         break;
       case ItemModifyJobPrivate::Dirty:
@@ -102,6 +114,9 @@ void ItemModifyJob::doStart()
     }
   }
 
+  if ( d->mItem.d_func()->mClearPayload )
+    changes << "INVALIDATECACHE.SILENT";
+
   if ( d->mItem.d_func()->mFlagsOverwritten ) {
     changes << "FLAGS.SILENT";
     changes << '(' + ImapParser::join( d->mItem.flags(), " " ) + ')';
@@ -139,7 +154,8 @@ void ItemModifyJob::doStart()
     command += "REV " + QByteArray::number( d->mItem.revision() ) + ' ';
   }
 
-  command += "SIZE " + QByteArray::number( d->mItem.size() );
+  if ( d->mItem.d_func()->mSizeChanged )
+    command += "SIZE " + QByteArray::number( d->mItem.size() );
 
   command += " (" + ImapParser::join( changes, " " );
   const QByteArray attrs = ProtocolHelper::attributesToByteArray( d->mItem, true );
@@ -160,19 +176,24 @@ void ItemModifyJob::doHandleResponse(const QByteArray &_tag, const QByteArray &
     return;
   }
   if ( _tag == d->mTag ) {
-    if ( data.startsWith( "OK" ) ) {
+    if ( data.startsWith( "OK" ) ) { //krazy:exclude=strings
       QDateTime modificationDateTime;
-      if ( int pos = data.indexOf( "DATETIME" ) ) {
-        int resultPos = ImapParser::parseDateTime( data, modificationDateTime, pos + 8 );
-        if ( resultPos == (pos + 8) ) {
-          kDebug( 5250 ) << "Invalid DATETIME response to STORE command: "
-              << _tag << data;
+      int dateTimePos = data.indexOf( "DATETIME" );
+      if ( dateTimePos != -1 ) {
+        int resultPos = ImapParser::parseDateTime( data, modificationDateTime, dateTimePos + 8 );
+        if ( resultPos == (dateTimePos + 8) ) {
+          kDebug() << "Invalid DATETIME response to STORE command: " << _tag << data;
         }
       }
 
-      // increase item revision of own copy of item
-      d->mItem.setRevision( d->mItem.revision() + 1 );
-      d->mItem.setModificationTime( modificationDateTime );
+      if ( d->mItem.modificationTime() != modificationDateTime )
+      {
+        // increase item revision of own copy of item
+        d->mItem.setRevision( d->mItem.revision() + 1 );
+        d->mItem.setModificationTime( modificationDateTime );
+      } else {
+        kDebug() << "No changes on item" << d->mItem.id();
+      }
       d->mItem.d_ptr->resetChangeLog();
     } else {
       setError( Unknown );
@@ -181,7 +202,7 @@ void ItemModifyJob::doHandleResponse(const QByteArray &_tag, const QByteArray &
     emitResult();
     return;
   }
-  kDebug( 5250 ) << "Unhandled response: " << _tag << data;
+  kDebug() << "Unhandled response: " << _tag << data;
 }
 
 void ItemModifyJob::setIgnorePayload( bool ignore )
