@@ -79,26 +79,49 @@ QByteArray ProtocolHelper::cachePolicyToByteArray(const CachePolicy & policy)
   return rv;
 }
 
+void ProtocolHelper::parseAncestorsCached( const QByteArray &data, Entity *entity, Collection::Id parentCollection,
+                                           ProtocolHelperValuePool *pool, int start )
+{
+  if ( !pool || parentCollection == -1 ) {
+    // if no pool or parent collection id is provided we can't cache anything, so continue as usual
+    parseAncestors( data, entity, start );
+    return;
+  }
+
+  if ( pool->ancestorCollections.contains( parentCollection ) ) {
+    // ancestor chain is cached already, so use the cached value
+    entity->setParentCollection( pool->ancestorCollections.value( parentCollection ) );
+  } else {
+    // not cached yet, parse the chain
+    parseAncestors( data, entity, start );
+    pool->ancestorCollections.insert( parentCollection, entity->parentCollection() );
+  }
+}
+
 void ProtocolHelper::parseAncestors( const QByteArray &data, Entity *entity, int start )
 {
   Q_UNUSED( start );
 
-  QList<QByteArray> ancestors;
+  static const Collection::Id rootCollectionId = Collection::root().id();
+  QVarLengthArray<QByteArray, 16> ancestors;
+  QVarLengthArray<QByteArray, 16> parentIds;
+
   ImapParser::parseParenthesizedList( data, ancestors );
   Entity* current = entity;
-  foreach ( const QByteArray &uidRidPair, ancestors ) {
-    QList<QByteArray> parentIds;
-    ImapParser::parseParenthesizedList( uidRidPair, parentIds );
+  for ( int i = 0; i < ancestors.count(); ++i ) {
+    parentIds.clear();
+    ImapParser::parseParenthesizedList( ancestors[ i ], parentIds );
     if ( parentIds.size() != 2 )
       break;
-    const Collection::Id uid = parentIds.at( 0 ).toLongLong();
-    const QString rid = QString::fromUtf8( parentIds.at( 1 ) );
-    if ( uid == Collection::root().id() ) {
+
+    const Collection::Id uid = parentIds[ 0 ].toLongLong();
+    if ( uid == rootCollectionId ) {
       current->setParentCollection( Collection::root() );
       break;
     }
+
     current->parentCollection().setId( uid );
-    current->parentCollection().setRemoteId( rid );
+    current->parentCollection().setRemoteId( QString::fromUtf8( parentIds[ 1 ] ) );
     current = &current->parentCollection();
   }
 }
@@ -270,7 +293,7 @@ QByteArray ProtocolHelper::itemFetchScopeToByteArray( const ItemFetchScope &fetc
   return command;
 }
 
-void ProtocolHelper::parseItemFetchResult( const QList<QByteArray> &lineTokens, Item &item )
+void ProtocolHelper::parseItemFetchResult( const QList<QByteArray> &lineTokens, Item &item, ProtocolHelperValuePool *valuePool )
 {
   // create a new item object
   Item::Id uid = -1;
@@ -298,7 +321,10 @@ void ProtocolHelper::parseItemFetchResult( const QList<QByteArray> &lineTokens,
     } else if ( key == "COLLECTIONID" ) {
       cid = value.toInt();
     } else if ( key == "MIMETYPE" ) {
-      mimeType = QString::fromLatin1( value );
+      if ( valuePool )
+        mimeType = valuePool->mimeTypePool.sharedValue( QString::fromLatin1( value ) );
+      else
+        mimeType = QString::fromLatin1( value );
     }
   }
 
@@ -327,8 +353,16 @@ void ProtocolHelper::parseItemFetchResult( const QList<QByteArray> &lineTokens,
     if ( key == "FLAGS" ) {
       QList<QByteArray> flags;
       ImapParser::parseParenthesizedList( lineTokens[i + 1], flags );
-      foreach ( const QByteArray &flag, flags ) {
-        item.setFlag( flag );
+      if ( !flags.isEmpty() ) {
+        Item::Flags convertedFlags;
+        convertedFlags.reserve( flags.size() );
+        foreach ( const QByteArray &flag, flags ) {
+          if ( valuePool )
+            convertedFlags.insert( valuePool->flagPool.sharedValue( flag ) );
+          else
+            convertedFlags.insert( flag );
+        }
+        item.setFlags( convertedFlags );
       }
     } else if ( key == "SIZE" ) {
       const quint64 size = lineTokens[i + 1].toLongLong();
@@ -338,7 +372,7 @@ void ProtocolHelper::parseItemFetchResult( const QList<QByteArray> &lineTokens,
       ImapParser::parseDateTime( lineTokens[i + 1], datetime );
       item.setModificationTime( datetime );
     } else if ( key == "ANCESTORS" ) {
-      ProtocolHelper::parseAncestors( lineTokens[i + 1], &item );
+      ProtocolHelper::parseAncestorsCached( lineTokens[i + 1], &item, cid, valuePool );
     } else {
       int version = 0;
       QByteArray plainKey( key );
