@@ -31,6 +31,7 @@
 
 #include "changerecorder.h"
 #include "collectionfetchjob.h"
+#include "collectionfetchscope.h"
 #include "collectionmodifyjob.h"
 #include "itemfetchjob.h"
 #include "itemfetchscope.h"
@@ -38,6 +39,8 @@
 #include "itemmodifyjob_p.h"
 #include "session.h"
 #include "resourceselectjob_p.h"
+#include "monitor_p.h"
+#include "servermanager_p.h"
 
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
@@ -56,29 +59,43 @@ using namespace Akonadi;
 
 class Akonadi::ResourceBasePrivate : public AgentBasePrivate
 {
+  Q_OBJECT
+  Q_CLASSINFO( "D-Bus Interface", "org.kde.dfaure" )
+
   public:
     ResourceBasePrivate( ResourceBase *parent )
       : AgentBasePrivate( parent ),
         scheduler( 0 ),
         mItemSyncer( 0 ),
-        mCollectionSyncer( 0 )
+        mCollectionSyncer( 0 ),
+        mHierarchicalRid( false )
     {
+      Internal::setClientType( Internal::Resource );
       mStatusMessage = defaultReadyMessage();
+
+      QDBusConnection::sessionBus().registerObject( QLatin1String( "/Debug" ), this, QDBusConnection::ExportScriptableSlots );
     }
 
     Q_DECLARE_PUBLIC( ResourceBase )
 
     void delayedInit()
     {
-      if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.freedesktop.Akonadi.Resource." ) + mId ) )
-        kFatal() << "Unable to register service at D-Bus: " << QDBusConnection::sessionBus().lastError().message();
-      AgentBasePrivate::delayedInit();
+      if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.freedesktop.Akonadi.Resource." ) + mId ) ) {
+        QString reason = QDBusConnection::sessionBus().lastError().message();
+        if ( reason.isEmpty() ) {
+          reason = QString::fromLatin1( "this service is probably running already." );
+        }
+        kError() << "Unable to register service at D-Bus: " << reason;
+        QCoreApplication::instance()->exit(1);
+      } else {
+        AgentBasePrivate::delayedInit();
+      }
     }
 
     virtual void changeProcessed()
     {
-      mMonitor->changeProcessed();
-      if ( !mMonitor->isEmpty() )
+      mChangeRecorder->changeProcessed();
+      if ( !mChangeRecorder->isEmpty() )
         scheduler->scheduleChangeReplay();
       scheduler->taskDone();
     }
@@ -96,12 +113,30 @@ class Akonadi::ResourceBasePrivate : public AgentBasePrivate
     void slotDeleteResourceCollectionDone( KJob *job );
     void slotCollectionDeletionDone( KJob *job );
 
+    void slotPrepareItemRetrieval( const Akonadi::Item &item );
+    void slotPrepareItemRetrievalResult( KJob* job );
+
+    void changeCommittedResult( KJob* job );
+
+  public Q_SLOTS:
+    Q_SCRIPTABLE void dump()
+    {
+      scheduler->dump();
+    }
+
+    Q_SCRIPTABLE void clear()
+    {
+      scheduler->clear();
+    }
+
+  public:
     // synchronize states
     Collection currentCollection;
 
     ResourceScheduler *scheduler;
     ItemSync *mItemSyncer;
     CollectionSync *mCollectionSyncer;
+    bool mHierarchicalRid;
 };
 
 ResourceBase::ResourceBase( const QString & id )
@@ -109,15 +144,15 @@ ResourceBase::ResourceBase( const QString & id )
 {
   Q_D( ResourceBase );
 
-  new ResourceAdaptor( this );
+  new Akonadi__ResourceAdaptor( this );
 
   d->scheduler = new ResourceScheduler( this );
 
-  d->mMonitor->setChangeRecordingEnabled( true );
-  connect( d->mMonitor, SIGNAL( changesAdded() ),
+  d->mChangeRecorder->setChangeRecordingEnabled( true );
+  connect( d->mChangeRecorder, SIGNAL( changesAdded() ),
            d->scheduler, SLOT( scheduleChangeReplay() ) );
 
-  d->mMonitor->setResourceMonitored( d->mId.toLatin1() );
+  d->mChangeRecorder->setResourceMonitored( d->mId.toLatin1() );
 
   connect( d->scheduler, SIGNAL( executeFullSync() ),
            SLOT( retrieveCollections() ) );
@@ -126,21 +161,23 @@ ResourceBase::ResourceBase( const QString & id )
   connect( d->scheduler, SIGNAL( executeCollectionSync( const Akonadi::Collection& ) ),
            SLOT( slotSynchronizeCollection( const Akonadi::Collection& ) ) );
   connect( d->scheduler, SIGNAL( executeItemFetch( const Akonadi::Item&, const QSet<QByteArray>& ) ),
-           SLOT( retrieveItem( const Akonadi::Item&, const QSet<QByteArray>& ) ) );
+           SLOT( slotPrepareItemRetrieval(Akonadi::Item)) );
   connect( d->scheduler, SIGNAL( executeResourceCollectionDeletion() ),
            SLOT( slotDeleteResourceCollection() ) );
   connect( d->scheduler, SIGNAL( status( int, const QString& ) ),
            SIGNAL( status( int, const QString& ) ) );
   connect( d->scheduler, SIGNAL( executeChangeReplay() ),
-           d->mMonitor, SLOT( replayNext() ) );
+           d->mChangeRecorder, SLOT( replayNext() ) );
   connect( d->scheduler, SIGNAL( fullSyncComplete() ), SIGNAL( synchronized() ) );
-  connect( d->mMonitor, SIGNAL( nothingToReplay() ), d->scheduler, SLOT( taskDone() ) );
+  connect( d->mChangeRecorder, SIGNAL( nothingToReplay() ), d->scheduler, SLOT( taskDone() ) );
+  connect( d->mChangeRecorder, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
+           d->scheduler, SLOT( collectionRemoved( const Akonadi::Collection& ) ) );
   connect( this, SIGNAL( synchronized() ), d->scheduler, SLOT( taskDone() ) );
   connect( this, SIGNAL( agentNameChanged( const QString& ) ),
            this, SIGNAL( nameChanged( const QString& ) ) );
 
   d->scheduler->setOnline( d->mOnline );
-  if ( !d->mMonitor->isEmpty() )
+  if ( !d->mChangeRecorder->isEmpty() )
     d->scheduler->scheduleChangeReplay();
 
   new ResourceSelectJob( identifier() );
@@ -169,7 +206,7 @@ QString ResourceBase::parseArguments( int argc, char **argv )
 {
   QString identifier;
   if ( argc < 3 ) {
-    kDebug( 5250 ) << "Not enough arguments passed...";
+    kDebug() << "Not enough arguments passed...";
     exit( 1 );
   }
 
@@ -179,7 +216,7 @@ QString ResourceBase::parseArguments( int argc, char **argv )
   }
 
   if ( identifier.isEmpty() ) {
-    kDebug( 5250 ) << "Identifier argument missing";
+    kDebug() << "Identifier argument missing";
     exit( 1 );
   }
 
@@ -191,12 +228,12 @@ QString ResourceBase::parseArguments( int argc, char **argv )
     catalog = QByteArray( argv[0] );
 
   KCmdLineArgs::init( argc, argv, identifier.toLatin1(), catalog,
-                      ki18nc("@title, application name", "Akonadi Resource"), "0.1",
-                      ki18nc("@title, application description", "Akonadi Resource") );
+                      ki18nc( "@title application name", "Akonadi Resource" ), "0.1",
+                      ki18nc( "@title application description", "Akonadi Resource" ) );
 
   KCmdLineOptions options;
   options.add( "identifier <argument>",
-               ki18nc("@label, commandline option", "Resource identifier") );
+               ki18nc( "@label commandline option", "Resource identifier" ) );
   KCmdLineArgs::addCmdLineOptions( options );
 
   return identifier;
@@ -215,9 +252,7 @@ void ResourceBase::itemRetrieved( const Item &item )
   Q_D( ResourceBase );
   Q_ASSERT( d->scheduler->currentTask().type == ResourceScheduler::FetchItem );
   if ( !item.isValid() ) {
-    QDBusMessage reply( d->scheduler->currentTask().dbusMsg );
-    reply << false;
-    QDBusConnection::sessionBus().send( reply );
+    d->scheduler->currentTask().sendDBusReplies( false );
     d->scheduler->taskDone();
     return;
   }
@@ -226,7 +261,7 @@ void ResourceBase::itemRetrieved( const Item &item )
   QSet<QByteArray> requestedParts = d->scheduler->currentTask().itemParts;
   foreach ( const QByteArray &part, requestedParts ) {
     if ( !item.loadedPayloadParts().contains( part ) ) {
-      kWarning( 5250 ) << "Item does not provide part" << part;
+      kWarning() << "Item does not provide part" << part;
     }
   }
 
@@ -240,14 +275,10 @@ void ResourceBasePrivate::slotDeliveryDone(KJob * job)
 {
   Q_Q( ResourceBase );
   Q_ASSERT( scheduler->currentTask().type == ResourceScheduler::FetchItem );
-  QDBusMessage reply( scheduler->currentTask().dbusMsg );
   if ( job->error() ) {
     emit q->error( QLatin1String( "Error while creating item: " ) + job->errorString() );
-    reply << false;
-  } else {
-    reply << true;
   }
-  QDBusConnection::sessionBus().send( reply );
+  scheduler->currentTask().sendDBusReplies( !job->error() );
   scheduler->taskDone();
 }
 
@@ -256,7 +287,7 @@ void ResourceBasePrivate::slotDeleteResourceCollection()
   Q_Q( ResourceBase );
 
   CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::FirstLevel );
-  job->setResource( q->identifier() );
+  job->fetchScope().setResource( q->identifier() );
   connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotDeleteResourceCollectionDone( KJob* ) ) );
 }
 
@@ -301,11 +332,17 @@ void ResourceBase::changeCommitted( const Item& item )
 
 void ResourceBase::changeCommitted( const Collection &collection )
 {
-  Q_D( ResourceBase );
   CollectionModifyJob *job = new CollectionModifyJob( collection );
-  Q_UNUSED( job );
-  //TODO: error checking
-  d->changeProcessed();
+  connect( job, SIGNAL( result( KJob* ) ), SLOT( changeCommittedResult( KJob* ) ) );
+}
+
+void ResourceBasePrivate::changeCommittedResult( KJob *job )
+{
+  Q_Q( ResourceBase );
+  if ( job->error() )
+    emit q->error( i18nc( "@info", "Updating local collection failed: %1.", job->errorText() ) );
+  mChangeRecorder->d_ptr->invalidateCache( static_cast<CollectionModifyJob*>( job )->collection() );
+  changeProcessed();
 }
 
 bool ResourceBase::requestItemDelivery( qint64 uid, const QString & remoteId,
@@ -341,6 +378,7 @@ void ResourceBase::collectionsRetrieved( const Collection::List & collections )
               "Calling collectionsRetrieved() although no collection retrieval is in progress" );
   if ( !d->mCollectionSyncer ) {
     d->mCollectionSyncer = new CollectionSync( identifier() );
+    d->mCollectionSyncer->setHierarchicalRemoteIds( d->mHierarchicalRid );
     connect( d->mCollectionSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
     connect( d->mCollectionSyncer, SIGNAL( result( KJob* ) ), SLOT( slotCollectionSyncDone( KJob* ) ) );
   }
@@ -357,6 +395,7 @@ void ResourceBase::collectionsRetrievedIncremental( const Collection::List & cha
               "Calling collectionsRetrievedIncremental() although no collection retrieval is in progress" );
   if ( !d->mCollectionSyncer ) {
     d->mCollectionSyncer = new CollectionSync( identifier() );
+    d->mCollectionSyncer->setHierarchicalRemoteIds( d->mHierarchicalRid );
     connect( d->mCollectionSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
     connect( d->mCollectionSyncer, SIGNAL( result( KJob* ) ), SLOT( slotCollectionSyncDone( KJob* ) ) );
   }
@@ -372,6 +411,7 @@ void ResourceBase::setCollectionStreamingEnabled( bool enable )
               "Calling setCollectionStreamingEnabled() although no collection retrieval is in progress" );
   if ( !d->mCollectionSyncer ) {
     d->mCollectionSyncer = new CollectionSync( identifier() );
+    d->mCollectionSyncer->setHierarchicalRemoteIds( d->mHierarchicalRid );
     connect( d->mCollectionSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
     connect( d->mCollectionSyncer, SIGNAL( result( KJob* ) ), SLOT( slotCollectionSyncDone( KJob* ) ) );
   }
@@ -391,6 +431,7 @@ void ResourceBase::collectionsRetrievalDone()
   }
   // user did the sync himself, we are done now
   else {
+    // FIXME: we need the same special case for SyncAll as in slotCollectionSyncDone here!
     d->scheduler->taskDone();
   }
 }
@@ -400,11 +441,13 @@ void ResourceBasePrivate::slotCollectionSyncDone( KJob * job )
   Q_Q( ResourceBase );
   mCollectionSyncer = 0;
   if ( job->error() ) {
-    emit q->error( job->errorString() );
+    if ( job->error() != Job::UserCanceled )
+      emit q->error( job->errorString() );
   } else {
     if ( scheduler->currentTask().type == ResourceScheduler::SyncAll ) {
       CollectionFetchJob *list = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive );
-      list->setResource( mId );
+      list->setFetchScope( q->changeRecorder()->collectionFetchScope() );
+      list->fetchScope().setResource( mId );
       q->connect( list, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalListDone( KJob* ) ) );
       return;
     }
@@ -434,7 +477,7 @@ void ResourceBasePrivate::slotSynchronizeCollection( const Collection &col )
   // check if this collection actually can contain anything
   QStringList contentTypes = currentCollection.contentMimeTypes();
   contentTypes.removeAll( Collection::mimeType() );
-  if ( !contentTypes.isEmpty() ) {
+  if ( !contentTypes.isEmpty() || (col.rights() & (Collection::CanLinkItem)) ) { // HACK to check for virtual collections
     emit q->status( AgentBase::Running, i18nc( "@info:status", "Syncing collection '%1'", currentCollection.name() ) );
     q->retrieveItems( currentCollection );
     return;
@@ -442,6 +485,42 @@ void ResourceBasePrivate::slotSynchronizeCollection( const Collection &col )
   scheduler->taskDone();
 }
 
+void ResourceBasePrivate::slotPrepareItemRetrieval( const Akonadi::Item &item )
+{
+  Q_Q( ResourceBase );
+  ItemFetchJob *fetch = new ItemFetchJob( item, this );
+  fetch->fetchScope().setAncestorRetrieval( q->changeRecorder()->itemFetchScope().ancestorRetrieval() );
+  fetch->fetchScope().setCacheOnly( true );
+
+  // copy list of attributes to fetch
+  const QSet<QByteArray> attributes = q->changeRecorder()->itemFetchScope().attributes();
+  foreach ( const QByteArray &attribute, attributes )
+    fetch->fetchScope().fetchAttribute( attribute );
+
+  q->connect( fetch, SIGNAL( result( KJob* ) ), SLOT( slotPrepareItemRetrievalResult( KJob* ) ) );
+}
+
+void ResourceBasePrivate::slotPrepareItemRetrievalResult( KJob* job )
+{
+  Q_Q( ResourceBase );
+  Q_ASSERT_X( scheduler->currentTask().type == ResourceScheduler::FetchItem,
+            "ResourceBasePrivate::slotPrepareItemRetrievalResult()",
+            "Preparing item retrieval although no item retrieval is in progress" );
+  if ( job->error() ) {
+    q->cancelTask( job->errorText() );
+    return;
+  }
+  ItemFetchJob *fetch = qobject_cast<ItemFetchJob*>( job );
+  if ( fetch->items().count() != 1 ) {
+    q->cancelTask( i18n( "The requested item no longer exists" ) );
+    return;
+  }
+  const Item item = fetch->items().first();
+  const QSet<QByteArray> parts = scheduler->currentTask().itemParts;
+  if ( !q->retrieveItem( item, parts ) )
+    q->cancelTask();
+}
+
 void ResourceBase::itemsRetrievalDone()
 {
   Q_D( ResourceBase );
@@ -494,6 +573,19 @@ void ResourceBase::cancelTask()
     case ResourceScheduler::ChangeReplay:
       d->changeProcessed();
       break;
+    case ResourceScheduler::SyncCollectionTree:
+    case ResourceScheduler::SyncAll:
+      if ( d->mCollectionSyncer )
+        d->mCollectionSyncer->rollback();
+      else
+        d->scheduler->taskDone();
+      break;
+    case ResourceScheduler::SyncCollection:
+      if ( d->mItemSyncer )
+        d->mItemSyncer->rollback();
+      else
+        d->scheduler->taskDone();
+      break;
     default:
       d->scheduler->taskDone();
   }
@@ -520,7 +612,8 @@ void ResourceBase::doSetOnline( bool state )
 void ResourceBase::synchronizeCollection( qint64 collectionId )
 {
   CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), CollectionFetchJob::Base );
-  job->setResource( identifier() );
+  job->setFetchScope( changeRecorder()->collectionFetchScope() );
+  job->fetchScope().setResource( identifier() );
   connect( job, SIGNAL( result( KJob* ) ), SLOT( slotCollectionListDone( KJob* ) ) );
 }
 
@@ -590,7 +683,7 @@ void ResourceBasePrivate::slotItemSyncDone( KJob *job )
 {
   mItemSyncer = 0;
   Q_Q( ResourceBase );
-  if ( job->error() ) {
+  if ( job->error() && job->error() != Job::UserCanceled ) {
     emit q->error( job->errorString() );
   }
   scheduler->taskDone();
@@ -603,4 +696,23 @@ void ResourceBasePrivate::slotPercent( KJob *job, unsigned long percent )
   emit q->percent( percent );
 }
 
+void ResourceBase::setHierarchicalRemoteIdentifiersEnabled( bool enable )
+{
+  Q_D( ResourceBase );
+  d->mHierarchicalRid = enable;
+}
+
+void ResourceBase::scheduleCustomTask( QObject *receiver, const char *method, const QVariant &argument, SchedulePriority priority )
+{
+  Q_D( ResourceBase );
+  d->scheduler->scheduleCustomTask( receiver, method, argument, priority );
+}
+
+void ResourceBase::taskDone()
+{
+  Q_D( ResourceBase );
+  d->scheduler->taskDone();
+}
+
 #include "resourcebase.moc"
+#include "moc_resourcebase.cpp"
