@@ -31,6 +31,7 @@
 #include <KMessageBox>
 #include <KRun>
 #include <KStandardDirs>
+#include <KUser>
 
 #include <QtCore/QFileInfo>
 #include <QtCore/QProcess>
@@ -45,9 +46,9 @@
 
 // @cond PRIVATE
 
-#define AKONADI_CONTROL_SERVICE QLatin1String("org.freedesktop.Akonadi.Control")
-#define AKONADI_SERVER_SERVICE QLatin1String("org.freedesktop.Akonadi")
-#define AKONADI_SEARCH_SERVICE QLatin1String("org.kde.nepomuk.services.nepomukqueryservice")
+#define AKONADI_CONTROL_SERVICE QLatin1String( "org.freedesktop.Akonadi.Control" )
+#define AKONADI_SERVER_SERVICE QLatin1String( "org.freedesktop.Akonadi" )
+#define AKONADI_SEARCH_SERVICE QLatin1String( "org.kde.nepomuk.services.nepomukqueryservice" )
 
 using namespace Akonadi;
 
@@ -71,21 +72,22 @@ SelfTestDialog::SelfTestDialog(QWidget * parent) :
   setCaption( i18n( "Akonadi Server Self-Test" ) );
   setButtons( Close | User1 | User2 );
   setButtonText( User1, i18n( "Save Report..." ) );
-  setButtonIcon( User1, KIcon( QString::fromLatin1("document-save") ) );
+  setButtonIcon( User1, KIcon( QString::fromLatin1( "document-save" ) ) );
   setButtonText( User2, i18n( "Copy Report to Clipboard" ) );
-  setButtonIcon( User2, KIcon( QString::fromLatin1("edit-copy") ) );
+  setButtonIcon( User2, KIcon( QString::fromLatin1( "edit-copy" ) ) );
   showButtonSeparator( true );
   ui.setupUi( mainWidget() );
 
   mTestModel = new QStandardItemModel( this );
   ui.testView->setModel( mTestModel );
-  connect( ui.testView->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-           SLOT(selectionChanged(QModelIndex)) );
-  connect( ui.detailsLabel, SIGNAL(linkActivated(QString)), SLOT(linkActivated(QString)) );
+  connect( ui.testView->selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ),
+           SLOT( selectionChanged( const QModelIndex& ) ) );
+  connect( ui.detailsLabel, SIGNAL( linkActivated( const QString& ) ), SLOT( linkActivated( const QString& ) ) );
 
-  connect( this, SIGNAL(user1Clicked()), SLOT(saveReport()) );
-  connect( this, SIGNAL(user2Clicked()), SLOT(copyReport()) );
+  connect( this, SIGNAL( user1Clicked() ), SLOT( saveReport() ) );
+  connect( this, SIGNAL( user2Clicked() ), SLOT( copyReport() ) );
 
+  connect( ServerManager::self(), SIGNAL( stateChanged( Akonadi::ServerManager::State ) ), SLOT( runTests() ) );
   runTests();
 }
 
@@ -99,17 +101,17 @@ QStandardItem* SelfTestDialog::report( ResultType type, const KLocalizedString &
   QStandardItem *item = new QStandardItem( summary.toString() );
   switch ( type ) {
     case Skip:
-      item->setIcon( KIcon( QString::fromLatin1("dialog-ok") ) );
+      item->setIcon( KIcon( QString::fromLatin1( "dialog-ok" ) ) );
       break;
     case Success:
-      item->setIcon( KIcon( QString::fromLatin1("dialog-ok-apply") ) );
+      item->setIcon( KIcon( QString::fromLatin1( "dialog-ok-apply" ) ) );
       break;
     case Warning:
-      item->setIcon( KIcon( QString::fromLatin1("dialog-warning") ) );
+      item->setIcon( KIcon( QString::fromLatin1( "dialog-warning" ) ) );
       break;
     case Error:
     default:
-      item->setIcon( KIcon( QString::fromLatin1("dialog-error") ) );
+      item->setIcon( KIcon( QString::fromLatin1( "dialog-error" ) ) );
   }
   item->setEditable( false );
   item->setWhatsThis( details.toString() );
@@ -133,12 +135,15 @@ void SelfTestDialog::selectionChanged(const QModelIndex &index )
 
 void SelfTestDialog::runTests()
 {
-  const QString driver = serverSetting( QLatin1String("General"), "Driver", QLatin1String("QMYSQL") ).toString();
+  mTestModel->clear();
+
+  const QString driver = serverSetting( QLatin1String( "General" ), "Driver", QLatin1String( "QMYSQL" ) ).toString();
   testSQLDriver();
   if (driver == QLatin1String( "QPSQL" )) {
     testPSQLServer();
   }
   else {
+    testRootUser();
     testMySQLServer();
     testMySQLServerLog();
     testMySQLServerConfig();
@@ -162,7 +167,7 @@ QVariant SelfTestDialog::serverSetting(const QString & group, const char *key, c
 
 bool SelfTestDialog::useStandaloneMysqlServer() const
 {
-  const QString driver = serverSetting( QLatin1String("General"), "Driver", QLatin1String("QMYSQL") ).toString();
+  const QString driver = serverSetting( QLatin1String( "General" ), "Driver", QLatin1String( "QMYSQL" ) ).toString();
   if ( driver != QLatin1String( "QMYSQL" ) )
     return false;
   const bool startServer = serverSetting( driver, "StartServer", true ).toBool();
@@ -184,7 +189,7 @@ bool SelfTestDialog::runProcess(const QString & app, const QStringList & args, Q
 
 void SelfTestDialog::testSQLDriver()
 {
-  const QString driver = serverSetting( QLatin1String("General"), "Driver", QLatin1String("QMYSQL") ).toString();
+  const QString driver = serverSetting( QLatin1String( "General" ), "Driver", QLatin1String( "QMYSQL" ) ).toString();
   const QStringList availableDrivers = QSqlDatabase::drivers();
   const KLocalizedString detailsOk = ki18n( "The QtSQL driver '%1' is required by your current Akonadi server configuration and was found on your system." )
       .subs( driver );
@@ -192,7 +197,7 @@ void SelfTestDialog::testSQLDriver()
       "The following drivers are installed: %2.\n"
       "Make sure the required driver is installed." )
       .subs( driver )
-      .subs( availableDrivers.join( QLatin1String(", ") ) );
+      .subs( availableDrivers.join( QLatin1String( ", " ) ) );
   QStandardItem *item = 0;
   if ( availableDrivers.contains( driver ) )
     item = report( Success, ki18n( "Database driver found." ), detailsOk );
@@ -209,13 +214,13 @@ void SelfTestDialog::testMySQLServer()
     return;
   }
 
-  const QString driver = serverSetting( QLatin1String("General"), "Driver", QLatin1String("QMYSQL") ).toString();
-  const QString serverPath = serverSetting( driver,  "ServerPath", QLatin1String("") ).toString(); // ### default?
+  const QString driver = serverSetting( QLatin1String( "General" ), "Driver", QLatin1String( "QMYSQL" ) ).toString();
+  const QString serverPath = serverSetting( driver,  "ServerPath", QLatin1String( "" ) ).toString(); // ### default?
 
-  const KLocalizedString details = ki18n( "You currently have configured Akonadi to use the MySQL server '%1'.\n"
+  const KLocalizedString details = ki18n( "You have currently configured Akonadi to use the MySQL server '%1'.\n"
       "Make sure you have the MySQL server installed, set the correct path and ensure you have the "
       "necessary read and execution rights on the server executable. The server executable is typically "
-      "called 'mysqld', its locations varies depending on the distribution." ).subs( serverPath );
+      "called 'mysqld'; its location varies depending on the distribution." ).subs( serverPath );
 
   QFileInfo info( serverPath );
   if ( !info.exists() )
@@ -224,7 +229,7 @@ void SelfTestDialog::testMySQLServer()
     report( Error, ki18n( "MySQL server not readable." ), details );
   else if ( !info.isExecutable() )
     report( Error, ki18n( "MySQL server not executable." ), details );
-  else if ( !serverPath.contains( QLatin1String("mysqld") ) )
+  else if ( !serverPath.contains( QLatin1String( "mysqld" ) ) )
     report( Warning, ki18n( "MySQL found with unexpected name." ), details );
   else
     report( Success, ki18n( "MySQL server found." ), details );
@@ -254,7 +259,7 @@ void SelfTestDialog::testMySQLServerLog()
   const QFileInfo logFileInfo( logFileName );
   if ( !logFileInfo.exists() || logFileInfo.size() == 0 ) {
     report( Success, ki18n( "No current MySQL error log found." ),
-      ki18n( "The MySQL server did not report any errors during this startup into '%1'." ).subs( logFileName ) );
+      ki18n( "The MySQL server did not report any errors during this startup. The log can be found in '%1'." ).subs( logFileName ) );
     return;
   }
   QFile logFile( logFileName );
@@ -328,14 +333,14 @@ void SelfTestDialog::testMySQLServerConfig()
                   "Check your access rights." ).subs( makeLink( localConfig ) ) );
   }
 
-  const QString actualConfig = XdgBaseDirs::saveDir( "data", QLatin1String( "akonadi" ) ) + QLatin1String("/mysql.conf");
+  const QString actualConfig = XdgBaseDirs::saveDir( "data", QLatin1String( "akonadi" ) ) + QLatin1String( "/mysql.conf" );
   const QFileInfo actualConfigInfo( actualConfig );
   if ( actualConfig.isEmpty() || !actualConfigInfo.exists() || !actualConfigInfo.isReadable() ) {
     report( Error, ki18n( "MySQL server configuration not found or not readable." ),
             ki18n( "The MySQL server configuration was not found or is not readable." ) );
   } else {
     item = report( Success, ki18n( "MySQL server configuration is usable." ),
-                   ki18n( "The MySQL server configuration was found at %1 and is readable.").subs( makeLink( actualConfig ) ) );
+                   ki18n( "The MySQL server configuration was found at %1 and is readable." ).subs( makeLink( actualConfig ) ) );
     item->setData( actualConfig, FileIncludeRole );
   }
 }
@@ -344,15 +349,20 @@ void SelfTestDialog::testPSQLServer()
 {
   const QString dbname = serverSetting( QLatin1String( "QPSQL" ), "Name", QLatin1String( "akonadi" )).toString();
   const QString hostname = serverSetting( QLatin1String( "QPSQL" ), "Host", QLatin1String( "localhost" )).toString();
-  const QString username = serverSetting( QLatin1String( "QPSQL" ), "User", QLatin1String( "akonadi" )).toString();
-  const QString password = serverSetting( QLatin1String( "QPSQL" ), "Password", QLatin1String( "akonadi" )).toString();
+  const QString username = serverSetting( QLatin1String( "QPSQL" ), "User", QString() ).toString();
+  const QString password = serverSetting( QLatin1String( "QPSQL" ), "Password", QString() ).toString();
   const int port = serverSetting( QLatin1String( "QPSQL" ), "Port", 5432).toInt();
 
   QSqlDatabase db = QSqlDatabase::addDatabase( QLatin1String( "QPSQL" ) );
   db.setHostName( hostname );
   db.setDatabaseName( dbname );
-  db.setUserName( username );
-  db.setPassword( password );
+
+  if ( !username.isEmpty() )
+    db.setUserName( username );
+
+  if ( !password.isEmpty() )
+    db.setPassword( password );
+
   db.setPort( port );
 
   if ( !db.open() ) {
@@ -361,14 +371,14 @@ void SelfTestDialog::testPSQLServer()
   }
   else {
     report( Success, ki18n( "PostgreSQL server found." ),
-                   ki18n( "The PostgreSQL server was found and connection is working."));
+                   ki18n( "The PostgreSQL server was found and connection is working." ));
   }
   db.close();
 }
 
 void SelfTestDialog::testAkonadiCtl()
 {
-  const QString path = KStandardDirs::findExe( QLatin1String("akonadictl") );
+  const QString path = KStandardDirs::findExe( QLatin1String( "akonadictl" ) );
   if ( path.isEmpty() ) {
     report( Error, ki18n( "akonadictl not found" ),
                  ki18n( "The program 'akonadictl' needs to be accessible in $PATH. "
@@ -468,7 +478,7 @@ void SelfTestDialog::testResources()
   AgentType::List agentTypes = AgentManager::self()->types();
   bool resourceFound = false;
   foreach ( const AgentType &type, agentTypes ) {
-    if ( type.capabilities().contains( QLatin1String("Resource") ) ) {
+    if ( type.capabilities().contains( QLatin1String( "Resource" ) ) ) {
       resourceFound = true;
       break;
     }
@@ -483,9 +493,9 @@ void SelfTestDialog::testResources()
       ki18n( "No resource agents have been found, Akonadi is not usable without at least one. "
             "This usually means that no resource agents are installed or that there is a setup problem. "
             "The following paths have been searched: '%1'. "
-            "The XDG_DATA_DIRS environment variable is set to '%2', make sure this includes all paths "
-            "where Akonadi agents are installed to." )
-          .subs( pathList.join( QLatin1String(" ") ) )
+            "The XDG_DATA_DIRS environment variable is set to '%2'; make sure this includes all paths "
+            "where Akonadi agents are installed." )
+          .subs( pathList.join( QLatin1String( " " ) ) )
           .subs( QString::fromLocal8Bit( qgetenv( "XDG_DATA_DIRS" ) ) ) );
   }
   item->setData( pathList, ListDirectoryRole );
@@ -502,18 +512,18 @@ void Akonadi::SelfTestDialog::testServerLog()
                    ki18n( "The Akonadi server did not report any errors during its current startup." ) );
   } else {
     QStandardItem *item = report( Error, ki18n( "Current Akonadi server error log found." ),
-      ki18n( "The Akonadi server did report error during startup into %1." ).subs( makeLink( serverLog ) ) );
+      ki18n( "The Akonadi server reported errors during its current startup. The log can be found in %1." ).subs( makeLink( serverLog ) ) );
     item->setData( serverLog, FileIncludeRole );
   }
 
-  serverLog += QLatin1String(".old");
+  serverLog += QLatin1String( ".old" );
   info.setFile( serverLog );
   if ( !info.exists() || info.size() <= 0 ) {
     report( Success, ki18n( "No previous Akonadi server error log found." ),
                    ki18n( "The Akonadi server did not report any errors during its previous startup." ) );
   } else {
     QStandardItem *item = report( Error, ki18n( "Previous Akonadi server error log found." ),
-      ki18n( "The Akonadi server did report error during its previous startup into %1." ).subs( makeLink( serverLog ) ) );
+      ki18n( "The Akonadi server reported errors during its previous startup. The log can be found in %1." ).subs( makeLink( serverLog ) ) );
     item->setData( serverLog, FileIncludeRole );
   }
 }
@@ -528,23 +538,33 @@ void SelfTestDialog::testControlLog()
                    ki18n( "The Akonadi control process did not report any errors during its current startup." ) );
   } else {
     QStandardItem *item = report( Error, ki18n( "Current Akonadi control error log found." ),
-      ki18n( "The Akonadi control process did report error during startup into %1." ).subs( makeLink( controlLog ) ) );
+      ki18n( "The Akonadi control process reported errors during its current startup. The log can be found in %1." ).subs( makeLink( controlLog ) ) );
     item->setData( controlLog, FileIncludeRole );
   }
 
-  controlLog += QLatin1String(".old");
+  controlLog += QLatin1String( ".old" );
   info.setFile( controlLog );
   if ( !info.exists() || info.size() <= 0 ) {
     report( Success, ki18n( "No previous Akonadi control error log found." ),
                    ki18n( "The Akonadi control process did not report any errors during its previous startup." ) );
   } else {
     QStandardItem *item = report( Error, ki18n( "Previous Akonadi control error log found." ),
-      ki18n( "The Akonadi control process did report error during its previous startup into %1." ).subs( makeLink( controlLog ) ) );
+      ki18n( "The Akonadi control process reported errors during its previous startup. The log can be found in %1." ).subs( makeLink( controlLog ) ) );
     item->setData( controlLog, FileIncludeRole );
   }
 }
 
 
+void SelfTestDialog::testRootUser()
+{
+  KUser user;
+  if ( user.isSuperUser() ) {
+    report( Error, ki18n( "Akonadi was started as root" ), ki18n( "Running Internet-facing applications as root/administrator exposes you to many security risks. MySQL, used by this Akonadi installation, will not allow itself to run as root, to protect you from these risks." ) );
+  } else {
+    report( Success, ki18n( "Akonadi is not running as root" ), ki18n( "Akonadi is not running as a root/administrator user, which is the recommended setup for a secure system." ) );
+  }
+}
+
 QString SelfTestDialog::createReport()
 {
   QString result;
@@ -610,7 +630,11 @@ QString SelfTestDialog::createReport()
 
 void SelfTestDialog::saveReport()
 {
-  const QString fileName =  KFileDialog::getSaveFileName( KUrl(), QString(), this, i18n("Save Test Report") );
+  const QString defaultFileName = QLatin1String( "akonadi-selftest-report-" )
+                   + QDate::currentDate().toString( QLatin1String( "yyyyMMdd" ) )
+                   + QLatin1String( ".txt" );
+  const QString fileName =  KFileDialog::getSaveFileName( defaultFileName, QString(), this,
+                                                          i18n( "Save Test Report" ) );
   if ( fileName.isEmpty() )
     return;
 
@@ -631,7 +655,7 @@ void SelfTestDialog::copyReport()
 
 void SelfTestDialog::linkActivated(const QString & link)
 {
-  KRun::runUrl( KUrl::fromPath( link ), QLatin1String("text/plain"), this );
+  KRun::runUrl( KUrl::fromPath( link ), QLatin1String( "text/plain" ), this );
 }
 
 // @endcond
