@@ -31,6 +31,7 @@
 #include "itemmodel.h"
 #include "pastehelper_p.h"
 #include "subscriptiondialog_p.h"
+#include "specialcollectionattribute_p.h"
 
 #include <KAction>
 #include <KActionCollection>
@@ -62,24 +63,24 @@ static const struct {
   const char* slot;
   bool isActionMenu;
 } actionData[] = {
-  { "akonadi_collection_create", I18N_NOOP("&New Folder..."), "folder-new", 0, SLOT(slotCreateCollection()), false },
-  { "akonadi_collection_copy", 0, "edit-copy", 0, SLOT(slotCopyCollections()), false },
-  { "akonadi_collection_delete", I18N_NOOP("&Delete Folder"), "edit-delete", 0, SLOT(slotDeleteCollection()), false },
-  { "akonadi_collection_sync", I18N_NOOP("&Synchronize Folder"), "view-refresh", Qt::Key_F5, SLOT(slotSynchronizeCollection()), false },
-  { "akonadi_collection_properties", I18N_NOOP("Folder &Properties"), "configure", 0, SLOT(slotCollectionProperties()), false },
-  { "akonadi_item_copy", 0, "edit-copy", 0, SLOT(slotCopyItems()), false },
-  { "akonadi_paste", I18N_NOOP("&Paste"), "edit-paste", Qt::CTRL + Qt::Key_V, SLOT(slotPaste()), false },
-  { "akonadi_item_delete", 0, "edit-delete", Qt::Key_Delete, SLOT(slotDeleteItems()), false },
-  { "akonadi_manage_local_subscriptions", I18N_NOOP("Manage Local &Subscriptions..."), 0, 0, SLOT(slotLocalSubscription()), false },
-  { "akonadi_collection_add_to_favorites", I18N_NOOP("Add to Favorite Folders"), "bookmark-new", 0, SLOT(slotAddToFavorites()), false },
-  { "akonadi_collection_remove_from_favorites", I18N_NOOP("Remove from Favorite Folders"), "edit-delete", 0, SLOT(slotRemoveFromFavorites()), false },
-  { "akonadi_collection_rename_favorite", I18N_NOOP("Rename Favorite..."), "edit-rename", 0, SLOT(slotRenameFavorite()), false },
-  { "akonadi_collection_copy_to_menu", I18N_NOOP("Copy Folder To..."), "edit-copy", 0, SLOT(slotCopyCollectionTo(QAction*)), true },
-  { "akonadi_item_copy_to_menu", I18N_NOOP("Copy Item To..."), "edit-copy", 0, SLOT(slotCopyItemTo(QAction*)), true },
-  { "akonadi_item_move_to_menu", I18N_NOOP("Move Item To..."), "go-jump", 0, SLOT(slotMoveItemTo(QAction*)), true },
-  { "akonadi_collection_move_to_menu", I18N_NOOP("Move Folder To..."), "go-jump", 0, SLOT(slotMoveCollectionTo(QAction*)), true },
-  { "akonadi_item_cut", I18N_NOOP("&Cut Item"), "edit-cut", Qt::CTRL + Qt::Key_X, SLOT(slotCutItems()), false },
-  { "akonadi_collection_cut", I18N_NOOP("&Cut Folder"), "edit-cut", Qt::CTRL + Qt::Key_X, SLOT(slotCutCollections()), false }
+  { "akonadi_collection_create", I18N_NOOP( "&New Folder..." ), "folder-new", 0, SLOT( slotCreateCollection() ), false },
+  { "akonadi_collection_copy", 0, "edit-copy", 0, SLOT( slotCopyCollections() ), false },
+  { "akonadi_collection_delete", I18N_NOOP( "&Delete Folder" ), "edit-delete", 0, SLOT( slotDeleteCollection() ), false },
+  { "akonadi_collection_sync", I18N_NOOP( "&Synchronize Folder" ), "view-refresh", Qt::Key_F5, SLOT( slotSynchronizeCollection() ), false },
+  { "akonadi_collection_properties", I18N_NOOP( "Folder &Properties" ), "configure", 0, SLOT( slotCollectionProperties() ), false },
+  { "akonadi_item_copy", 0, "edit-copy", 0, SLOT( slotCopyItems() ), false },
+  { "akonadi_paste", I18N_NOOP( "&Paste" ), "edit-paste", Qt::CTRL + Qt::Key_V, SLOT( slotPaste() ), false },
+  { "akonadi_item_delete", 0, "edit-delete", Qt::Key_Delete, SLOT( slotDeleteItems() ), false },
+  { "akonadi_manage_local_subscriptions", I18N_NOOP( "Manage Local &Subscriptions..." ), 0, 0, SLOT( slotLocalSubscription() ), false },
+  { "akonadi_collection_add_to_favorites", I18N_NOOP( "Add to Favorite Folders" ), "bookmark-new", 0, SLOT( slotAddToFavorites() ), false },
+  { "akonadi_collection_remove_from_favorites", I18N_NOOP( "Remove from Favorite Folders" ), "edit-delete", 0, SLOT( slotRemoveFromFavorites() ), false },
+  { "akonadi_collection_rename_favorite", I18N_NOOP( "Rename Favorite..." ), "edit-rename", 0, SLOT( slotRenameFavorite() ), false },
+  { "akonadi_collection_copy_to_menu", I18N_NOOP( "Copy Folder To..." ), "edit-copy", 0, SLOT( slotCopyCollectionTo( QAction* ) ), true },
+  { "akonadi_item_copy_to_menu", I18N_NOOP( "Copy Item To..." ), "edit-copy", 0, SLOT( slotCopyItemTo( QAction* ) ), true },
+  { "akonadi_item_move_to_menu", I18N_NOOP( "Move Item To..." ), "go-jump", 0, SLOT( slotMoveItemTo( QAction* ) ), true },
+  { "akonadi_collection_move_to_menu", I18N_NOOP( "Move Folder To..." ), "go-jump", 0, SLOT( slotMoveCollectionTo( QAction* ) ), true },
+  { "akonadi_item_cut", I18N_NOOP( "&Cut Item" ), "edit-cut", Qt::CTRL + Qt::Key_X, SLOT( slotCutItems() ), false },
+  { "akonadi_collection_cut", I18N_NOOP( "&Cut Folder" ), "edit-cut", Qt::CTRL + Qt::Key_X, SLOT( slotCutCollections() ), false }
 };
 static const int numActionData = sizeof actionData / sizeof *actionData;
 
@@ -150,77 +151,69 @@ class StandardActionManager::Private
       }
     }
 
-    void encodeToClipboard( QItemSelectionModel* selModel, bool cut = false )
+    void encodeToClipboard( QItemSelectionModel* selectionModel, bool cut = false )
     {
-      Q_ASSERT( selModel );
-      if ( selModel->selectedRows().count() <= 0 )
+      Q_ASSERT( selectionModel );
+      if ( selectionModel->selectedRows().count() <= 0 )
         return;
-      QMimeData *mimeData = selModel->model()->mimeData( selModel->selectedRows() );
+
+      QMimeData *mimeData = selectionModel->model()->mimeData( selectionModel->selectedRows() );
       markCutAction( mimeData, cut );
       QApplication::clipboard()->setMimeData( mimeData );
 
-      QAbstractItemModel *model = const_cast<QAbstractItemModel *>( selModel->model() );
+      QAbstractItemModel *model = const_cast<QAbstractItemModel *>( selectionModel->model() );
 
-      foreach( const QModelIndex &index, selModel->selectedRows() )
-      {
+      foreach ( const QModelIndex &index, selectionModel->selectedRows() )
         model->setData( index, true, EntityTreeModel::PendingCutRole );
-      }
     }
 
     void updateActions()
     {
-      bool singleColSelected = false;
-      bool multiColSelected = false;
+      bool singleCollectionSelected = false;
+      bool multipleCollectionsSelected = false;
       bool canDeleteCollections = true;
-      int colCount = 0;
-
+      int collectionCount = 0;
       QModelIndex selectedIndex;
       if ( !collectionSelectionModel ) {
         canDeleteCollections = false;
       } else {
-        colCount = collectionSelectionModel->selectedRows().count();
-        singleColSelected = colCount == 1;
-        multiColSelected = colCount > 1;
-        canDeleteCollections = colCount > 0;
+        collectionCount = collectionSelectionModel->selectedRows().count();
+        singleCollectionSelected = collectionCount == 1;
+        multipleCollectionsSelected = collectionCount > 1;
+        canDeleteCollections = collectionCount > 0;
 
-        if ( singleColSelected )
+        if ( singleCollectionSelected )
           selectedIndex = collectionSelectionModel->selectedRows().first();
-
         if ( itemSelectionModel ) {
           const QModelIndexList rows = itemSelectionModel->selectedRows();
           foreach ( const QModelIndex &itemIndex, rows ) {
             const Collection collection = itemIndex.data( EntityTreeModel::CollectionRole ).value<Collection>();
             if ( !collection.isValid() )
               continue;
-
             if ( collection == collection.root() )
               // The root collection is selected. There are no valid actions to enable.
               return;
-
             canDeleteCollections = canDeleteCollections && ( collection.rights() & Collection::CanDeleteCollection );
           }
         }
       }
+      const Collection collection = selectedIndex.data( CollectionModel::CollectionRole ).value<Collection>();
 
-      Collection col = selectedIndex.data( CollectionModel::CollectionRole ).value<Collection>();
+      enableAction( CopyCollections, (singleCollectionSelected || multipleCollectionsSelected) && !isRootCollection( collection ) );
+      enableAction( CollectionProperties, singleCollectionSelected && !isRootCollection( collection ) );
 
-      enableAction( CopyCollections, (singleColSelected || multiColSelected) && !isRootCollection( col ) );
-      enableAction( CollectionProperties, singleColSelected && !isRootCollection( col ) );
+      enableAction( CreateCollection, singleCollectionSelected && canCreateCollection( collection ) );
+      enableAction( DeleteCollections, singleCollectionSelected && (collection.rights() & Collection::CanDeleteCollection) && !CollectionUtils::isResource( collection ) );
+      enableAction( CutCollections, canDeleteCollections && !isRootCollection( collection ) && !CollectionUtils::isResource( collection ) && CollectionUtils::isFolder( collection ) && !collection.hasAttribute<SpecialCollectionAttribute>() );
+      enableAction( SynchronizeCollections, singleCollectionSelected && (CollectionUtils::isResource( collection ) || CollectionUtils::isFolder( collection ) ) );
+      enableAction( Paste, singleCollectionSelected && PasteHelper::canPaste( QApplication::clipboard()->mimeData(), collection ) );
+      enableAction( AddToFavoriteCollections, singleCollectionSelected && ( favoritesModel != 0 ) && ( !favoritesModel->collections().contains( collection ) ) && !isRootCollection( collection ) && !CollectionUtils::isResource( collection ) && CollectionUtils::isFolder( collection ) );
+      enableAction( RemoveFromFavoriteCollections, singleCollectionSelected && ( favoritesModel != 0 ) && ( favoritesModel->collections().contains( collection ) ) );
+      enableAction( RenameFavoriteCollection, singleCollectionSelected && ( favoritesModel != 0 ) && ( favoritesModel->collections().contains( collection ) ) );
+      enableAction( CopyCollectionToMenu, (singleCollectionSelected || multipleCollectionsSelected) && !isRootCollection( collection ) );
+      enableAction( MoveCollectionToMenu, canDeleteCollections && !isRootCollection( collection ) && !CollectionUtils::isResource( collection ) && CollectionUtils::isFolder( collection ) && !collection.hasAttribute<SpecialCollectionAttribute>() );
 
-      enableAction( CreateCollection, singleColSelected && canCreateCollection( col ) );
-      enableAction( DeleteCollections, singleColSelected && (col.rights() & Collection::CanDeleteCollection) && !CollectionUtils::isResource( col ) );
-      enableAction( CopyCollections, (singleColSelected || multiColSelected) && !isRootCollection( col ) );
-      enableAction( CutCollections, canDeleteCollections && !isRootCollection( col ) && !CollectionUtils::isResource( col ) );
-      enableAction( CollectionProperties, singleColSelected && !isRootCollection( col ) );
-      enableAction( SynchronizeCollections, singleColSelected && (CollectionUtils::isResource( col ) || CollectionUtils::isFolder( col ) ) );
-      enableAction( Paste, singleColSelected && PasteHelper::canPaste( QApplication::clipboard()->mimeData(), col ) );
-      enableAction( AddToFavoriteCollections, singleColSelected && ( favoritesModel != 0 ) && ( !favoritesModel->collections().contains( col ) ) );
-      enableAction( RemoveFromFavoriteCollections, singleColSelected && ( favoritesModel != 0 ) && ( favoritesModel->collections().contains( col ) ) );
-      enableAction( RenameFavoriteCollection, singleColSelected && ( favoritesModel != 0 ) && ( favoritesModel->collections().contains( col ) ) );
-      enableAction( CopyCollectionToMenu, (singleColSelected || multiColSelected) && !isRootCollection( col ) );
-      enableAction( MoveCollectionToMenu, canDeleteCollections && !isRootCollection( col ) && !CollectionUtils::isResource( col ) );
-
-      bool multiItemSelected = false;
+      bool multipleItemsSelected = false;
       bool canDeleteItems = true;
       int itemCount = 0;
       if ( !itemSelectionModel ) {
@@ -229,7 +222,7 @@ class StandardActionManager::Private
         const QModelIndexList rows = itemSelectionModel->selectedRows();
 
         itemCount = rows.count();
-        multiItemSelected = itemCount > 0;
+        multipleItemsSelected = itemCount > 0;
         canDeleteItems = itemCount > 0;
 
         foreach ( const QModelIndex &itemIndex, rows ) {
@@ -241,15 +234,15 @@ class StandardActionManager::Private
         }
       }
 
-      enableAction( CopyItems, multiItemSelected );
+      enableAction( CopyItems, multipleItemsSelected );
       enableAction( CutItems, canDeleteItems );
 
-      enableAction( DeleteItems, multiItemSelected && canDeleteItems );
+      enableAction( DeleteItems, multipleItemsSelected && canDeleteItems );
 
-      enableAction( CopyItemToMenu, multiItemSelected );
-      enableAction( MoveItemToMenu, multiItemSelected && canDeleteItems );
+      enableAction( CopyItemToMenu, multipleItemsSelected );
+      enableAction( MoveItemToMenu, multipleItemsSelected && canDeleteItems );
 
-      updatePluralLabel( CopyCollections, colCount );
+      updatePluralLabel( CopyCollections, collectionCount );
       updatePluralLabel( CopyItems, itemCount );
       updatePluralLabel( DeleteItems, itemCount );
       updatePluralLabel( CutItems, itemCount );
@@ -264,7 +257,7 @@ class StandardActionManager::Private
         updateActions();
     }
 
-    QItemSelection mapToEntityTreeModel( const QAbstractItemModel *model, const QItemSelection &selection )
+    QItemSelection mapToEntityTreeModel( const QAbstractItemModel *model, const QItemSelection &selection ) const
     {
       const QAbstractProxyModel *proxy = qobject_cast<const QAbstractProxyModel*>( model );
       if ( proxy ) {
@@ -274,11 +267,11 @@ class StandardActionManager::Private
       }
     }
 
-    QItemSelection mapFromEntityTreeModel( const QAbstractItemModel *model, const QItemSelection &selection )
+    QItemSelection mapFromEntityTreeModel( const QAbstractItemModel *model, const QItemSelection &selection ) const
     {
       const QAbstractProxyModel *proxy = qobject_cast<const QAbstractProxyModel*>( model );
       if ( proxy ) {
-        QItemSelection select = mapFromEntityTreeModel( proxy->sourceModel(), selection );
+        const QItemSelection select = mapFromEntityTreeModel( proxy->sourceModel(), selection );
         return proxy->mapSelectionFromSource( select );
       } else {
         return selection;
@@ -287,7 +280,7 @@ class StandardActionManager::Private
 
     void collectionSelectionChanged()
     {
-      q->blockSignals(true);
+      q->blockSignals( true );
 
       QItemSelection selection = collectionSelectionModel->selection();
       selection = mapToEntityTreeModel( collectionSelectionModel->model(), selection );
@@ -296,22 +289,24 @@ class StandardActionManager::Private
       if ( favoriteSelectionModel )
         favoriteSelectionModel->select( selection, QItemSelectionModel::ClearAndSelect );
 
-      q->blockSignals(false);
+      q->blockSignals( false );
 
       updateActions();
     }
 
     void favoriteSelectionChanged()
     {
-      q->blockSignals(true);
+      q->blockSignals( true );
 
       QItemSelection selection = favoriteSelectionModel->selection();
-      if ( selection.indexes().isEmpty() ) return;
+      if ( selection.indexes().isEmpty() )
+        return;
+
       selection = mapToEntityTreeModel( favoritesModel, selection );
       selection = mapFromEntityTreeModel( collectionSelectionModel->model(), selection );
-      collectionSelectionModel->select( selection, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
 
-      q->blockSignals(false);
+      collectionSelectionModel->select( selection, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
+      q->blockSignals( false );
 
       updateActions();
     }
@@ -324,26 +319,23 @@ class StandardActionManager::Private
 
       const QModelIndex index = collectionSelectionModel->selection().indexes().at( 0 );
       Q_ASSERT( index.isValid() );
-      const Collection collection = index.data( CollectionModel::CollectionRole ).value<Collection>();
-      Q_ASSERT( collection.isValid() );
+      const Collection parentCollection = index.data( CollectionModel::CollectionRole ).value<Collection>();
+      Q_ASSERT( parentCollection.isValid() );
 
-      if ( !canCreateCollection( collection ) )
+      if ( !canCreateCollection( parentCollection ) )
         return;
 
-      const QString name = KInputDialog::getText( i18nc( "@title:window", "New Folder"),
-                                                  i18nc( "@label:textbox, name of a thing", "Name"),
+      const QString name = KInputDialog::getText( i18nc( "@title:window", "New Folder" ),
+                                                  i18nc( "@label:textbox name of a thing", "Name" ),
                                                   QString(), 0, parentWidget );
       if ( name.isEmpty() )
         return;
-      Collection::Id parentId = index.data( CollectionModel::CollectionIdRole ).toLongLong();
-      if ( parentId <= 0 )
-        return;
 
-      Collection col;
-      col.setName( name );
-      col.parentCollection().setId( parentId );
-      CollectionCreateJob *job = new CollectionCreateJob( col );
-      q->connect( job, SIGNAL(result(KJob*)), q, SLOT(collectionCreationResult(KJob*)) );
+      Collection collection;
+      collection.setName( name );
+      collection.setParentCollection( parentCollection );
+      CollectionCreateJob *job = new CollectionCreateJob( collection );
+      q->connect( job, SIGNAL( result( KJob* ) ), q, SLOT( collectionCreationResult( KJob* ) ) );
     }
 
     void slotCopyCollections()
@@ -372,15 +364,16 @@ class StandardActionManager::Private
         text = i18n( "Do you really want to delete the search view '%1'?", index.data().toString() );
 
       if ( KMessageBox::questionYesNo( parentWidget, text,
-           i18n("Delete folder?"), KStandardGuiItem::del(), KStandardGuiItem::cancel(),
+           i18n( "Delete folder?" ), KStandardGuiItem::del(), KStandardGuiItem::cancel(),
            QString(), KMessageBox::Dangerous ) != KMessageBox::Yes )
         return;
-      const Collection::Id colId = index.data( CollectionModel::CollectionIdRole ).toLongLong();
-      if ( colId <= 0 )
+
+      const Collection::Id collectionId = index.data( CollectionModel::CollectionIdRole ).toLongLong();
+      if ( collectionId <= 0 )
         return;
 
-      CollectionDeleteJob *job = new CollectionDeleteJob( Collection( colId ), q );
-      q->connect( job, SIGNAL(result(KJob*)), q, SLOT(collectionDeletionResult(KJob*)) );
+      CollectionDeleteJob *job = new CollectionDeleteJob( Collection( collectionId ), q );
+      q->connect( job, SIGNAL( result( KJob* ) ), q, SLOT( collectionDeletionResult( KJob* ) ) );
     }
 
     void slotSynchronizeCollection()
@@ -391,10 +384,10 @@ class StandardActionManager::Private
 
       const QModelIndex index = collectionSelectionModel->selection().indexes().at( 0 );
       Q_ASSERT( index.isValid() );
-      const Collection col = index.data( CollectionModel::CollectionRole ).value<Collection>();
-      Q_ASSERT( col.isValid() );
+      const Collection collection = index.data( CollectionModel::CollectionRole ).value<Collection>();
+      Q_ASSERT( collection.isValid() );
 
-      AgentManager::self()->synchronizeCollection( col );
+      AgentManager::self()->synchronizeCollection( collection );
     }
 
     void slotCollectionProperties()
@@ -403,11 +396,11 @@ class StandardActionManager::Private
         return;
       const QModelIndex index = collectionSelectionModel->selection().indexes().at( 0 );
       Q_ASSERT( index.isValid() );
-      Collection col = index.data( CollectionModel::CollectionRole ).value<Collection>();
-      Q_ASSERT( col.isValid() );
+      const Collection collection = index.data( CollectionModel::CollectionRole ).value<Collection>();
+      Q_ASSERT( collection.isValid() );
 
-      CollectionPropertiesDialog* dlg = new CollectionPropertiesDialog( col, parentWidget );
-      dlg->setCaption(i18n("Properties of Folder %1",col.name()));
+      CollectionPropertiesDialog* dlg = new CollectionPropertiesDialog( collection, parentWidget );
+      dlg->setCaption( i18n( "Properties of Folder %1", collection.name() ) );
       dlg->show();
     }
 
@@ -442,19 +435,22 @@ class StandardActionManager::Private
     {
       if ( KMessageBox::questionYesNo( parentWidget,
            i18n( "Do you really want to delete all selected items?" ),
-           i18n("Delete?"), KStandardGuiItem::del(), KStandardGuiItem::cancel(),
+           i18n( "Delete?" ), KStandardGuiItem::del(), KStandardGuiItem::cancel(),
            QString(), KMessageBox::Dangerous ) != KMessageBox::Yes )
         return;
 
       Q_ASSERT( itemSelectionModel );
 
       // TODO: fix this once ItemModifyJob can handle item lists
+      Item::List items;
       foreach ( const QModelIndex &index, itemSelectionModel->selectedRows() ) {
         bool ok;
-        qlonglong id = index.data( ItemModel::IdRole ).toLongLong(&ok);
-        Q_ASSERT(ok);
-        new ItemDeleteJob( Item( id ), q );
+        const qlonglong id = index.data( ItemModel::IdRole ).toLongLong( &ok );
+        Q_ASSERT( ok );
+        items << Item( id );
       }
+
+      new ItemDeleteJob( items, q );
     }
 
     void slotLocalSubscription()
@@ -509,9 +505,9 @@ class StandardActionManager::Private
       Q_ASSERT( collection.isValid() );
 
       bool ok;
-      QString label = KInputDialog::getText( i18n( "Rename Favorite" ),
-                                             i18nc( "@label:textbox New name of the folder.", "Name:" ),
-                                             favoritesModel->favoriteLabel( collection ), &ok, parentWidget );
+      const QString label = KInputDialog::getText( i18n( "Rename Favorite" ),
+                                                   i18nc( "@label:textbox New name of the folder.", "Name:" ),
+                                                   favoritesModel->favoriteLabel( collection ), &ok, parentWidget );
       if ( !ok )
         return;
 
@@ -546,9 +542,9 @@ class StandardActionManager::Private
       if ( selectionModel->selectedRows().count() <= 0 )
         return;
 
-      QMimeData *mimeData = selectionModel->model()->mimeData( selectionModel->selectedRows() );
+      const QMimeData *mimeData = selectionModel->model()->mimeData( selectionModel->selectedRows() );
 
-      QModelIndex index = action->data().value<QModelIndex>();
+      const QModelIndex index = action->data().value<QModelIndex>();
 
       Q_ASSERT( index.isValid() );
 
@@ -559,83 +555,84 @@ class StandardActionManager::Private
     void collectionCreationResult( KJob *job )
     {
       if ( job->error() ) {
-        KMessageBox::error( parentWidget, i18n("Could not create folder: %1", job->errorString()),
-                            i18n("Folder creation failed") );
+        KMessageBox::error( parentWidget, i18n( "Could not create folder: %1", job->errorString()),
+                            i18n( "Folder creation failed" ) );
       }
     }
 
     void collectionDeletionResult( KJob *job )
     {
       if ( job->error() ) {
-        KMessageBox::error( parentWidget, i18n("Could not delete folder: %1", job->errorString()),
-                            i18n("Folder deletion failed") );
+        KMessageBox::error( parentWidget, i18n( "Could not delete folder: %1", job->errorString()),
+                            i18n( "Folder deletion failed" ) );
       }
     }
 
     void pasteResult( KJob *job )
     {
       if ( job->error() ) {
-        KMessageBox::error( parentWidget, i18n("Could not paste data: %1", job->errorString()),
-                            i18n("Paste failed") );
+        KMessageBox::error( parentWidget, i18n( "Could not paste data: %1", job->errorString()),
+                            i18n( "Paste failed" ) );
       }
     }
 
     void fillFoldersMenu( StandardActionManager::Type type, QMenu *menu,
                           const QAbstractItemModel *model, QModelIndex parentIndex )
     {
-      int rowCount = model->rowCount( parentIndex );
+      const int rowCount = model->rowCount( parentIndex );
 
       QModelIndexList list;
-      QSet<QString> mimetypes;
+      QSet<QString> mimeTypes;
 
-      const bool itemAction = ( type == CopyItemToMenu || type == MoveItemToMenu );
-      const bool collectionAction = ( type == CopyCollectionToMenu || type == MoveCollectionToMenu );
+      const bool isItemAction = ( type == CopyItemToMenu || type == MoveItemToMenu );
+      const bool isCollectionAction = ( type == CopyCollectionToMenu || type == MoveCollectionToMenu );
 
-      if ( itemAction )
-      {
+      if ( isItemAction ) {
         list = itemSelectionModel->selectedRows();
-        foreach(const QModelIndex &idx, list)
-        {
-          mimetypes << idx.data( EntityTreeModel::MimeTypeRole ).toString();
-        }
+        foreach ( const QModelIndex &index, list )
+          mimeTypes << index.data( EntityTreeModel::MimeTypeRole ).toString();
       }
 
-      if ( collectionAction )
-      {
+      if ( isCollectionAction ) {
         list = collectionSelectionModel->selectedRows();
-        foreach(const QModelIndex &idx, list)
-        {
-          Collection collection = idx.data( EntityTreeModel::CollectionRole ).value<Collection>();
+        foreach ( const QModelIndex &index, list ) {
+          const Collection collection = index.data( EntityTreeModel::CollectionRole ).value<Collection>();
 
           // The mimetypes that the selected collection can possibly contain
-          mimetypes = AgentManager::self()->type( collection.resource() ).mimeTypes().toSet();
+          mimeTypes = AgentManager::self()->instance( collection.resource() ).type().mimeTypes().toSet();
         }
       }
 
-      for ( int row = 0; row < rowCount; row++ ) {
-        QModelIndex index = model->index( row, 0, parentIndex );
-        Collection collection = model->data( index, CollectionModel::CollectionRole ).value<Collection>();
+      for ( int row = 0; row < rowCount; ++row ) {
+        const QModelIndex index = model->index( row, 0, parentIndex );
+        const Collection collection = model->data( index, CollectionModel::CollectionRole ).value<Collection>();
 
         if ( CollectionUtils::isVirtual( collection ) ) {
           continue;
         }
 
         QString label = model->data( index ).toString();
-        label.replace( QString::fromUtf8( "&" ), QString::fromUtf8( "&&" ) );
-        QIcon icon = model->data( index, Qt::DecorationRole ).value<QIcon>();
+        label.replace( QLatin1String( "&" ), QLatin1String( "&&" ) );
+
+        const QIcon icon = model->data( index, Qt::DecorationRole ).value<QIcon>();
+
+        const bool canContainRequiredMimeTypes = !collection.contentMimeTypes().toSet().intersect( mimeTypes ).isEmpty();
+        const bool canCreateNewItems = (collection.rights() & Collection::CanCreateItem);
 
 
-        bool readOnly = CollectionUtils::isStructural( collection )
-              || ( itemAction && ( !( collection.rights() & Collection::CanCreateItem )
-                    || collection.contentMimeTypes().toSet().intersect( mimetypes ).isEmpty() ) )
-              || ( collectionAction && ( !( collection.rights() & Collection::CanCreateCollection )
-                    || QSet<QString>( mimetypes ).subtract( AgentManager::self()->type( collection.resource() ).mimeTypes().toSet() ).isEmpty()
-                    || !collection.contentMimeTypes().contains( Collection::mimeType() ) ) );
+        const bool canCreateNewCollections = (collection.rights() & Collection::CanCreateCollection);
+        const bool canContainCollections = collection.contentMimeTypes().contains( Collection::mimeType() );
+        const bool resourceAllowsRequiredMimeTypes = AgentManager::self()->instance( collection.resource() ).type().mimeTypes().toSet().contains( mimeTypes );
+
+        const bool isReadOnlyForItems = (isItemAction && (!canCreateNewItems || !canContainRequiredMimeTypes));
+        const bool isReadOnlyForCollections = (isCollectionAction && (!canCreateNewCollections || !canContainCollections || !resourceAllowsRequiredMimeTypes));
+
+        const bool readOnly = CollectionUtils::isStructural( collection ) || isReadOnlyForItems || isReadOnlyForCollections;
 
         if ( model->rowCount( index ) > 0 ) {
           // new level
           QMenu* popup = new QMenu( menu );
-
+          const bool moveAction = ( type == MoveCollectionToMenu || type == MoveItemToMenu);
           popup->setObjectName( QString::fromUtf8( "subMenu" ) );
           popup->setTitle( label );
           popup->setIcon( icon );
@@ -645,31 +642,31 @@ class StandardActionManager::Private
           if ( !readOnly ) {
             popup->addSeparator();
 
-            QAction *act = popup->addAction( i18n("Copy to This Folder") );
-            act->setData( QVariant::fromValue<QModelIndex>( index ) );
+            QAction *action = popup->addAction( moveAction ? i18n( "Move to This Folder" ) : i18n( "Copy to This Folder" ) );
+            action->setData( QVariant::fromValue<QModelIndex>( index ) );
           }
 
           menu->addMenu( popup );
 
         } else {
           // insert an item
-          QAction* act = menu->addAction( icon, label );
-          act->setData( QVariant::fromValue<QModelIndex>( index ) );
-          act->setEnabled( !readOnly );
+          QAction* action = menu->addAction( icon, label );
+          action->setData( QVariant::fromValue<QModelIndex>( index ) );
+          action->setEnabled( !readOnly );
         }
       }
     }
 
     void checkModelsConsistency()
     {
-      if ( favoritesModel==0 || favoriteSelectionModel==0 ) {
+      if ( favoritesModel == 0 || favoriteSelectionModel == 0 ) {
         // No need to check when the favorite collections feature is not used
         return;
       }
 
       // Check that the collection selection model maps to the same
       // EntityTreeModel than favoritesModel
-      if ( collectionSelectionModel!=0 ) {
+      if ( collectionSelectionModel != 0 ) {
         const QAbstractItemModel *model = collectionSelectionModel->model();
         while ( const QAbstractProxyModel *proxy = qobject_cast<const QAbstractProxyModel*>( model ) ) {
           model = proxy->sourceModel();
@@ -696,11 +693,11 @@ class StandardActionManager::Private
 
     bool isCutAction( const QMimeData *mimeData ) const
     {
-      QByteArray a = mimeData->data( QLatin1String( "application/x-kde.akonadi-cutselection" ) );
-      if ( a.isEmpty() )
+      const QByteArray data = mimeData->data( QLatin1String( "application/x-kde.akonadi-cutselection" ) );
+      if ( data.isEmpty() )
         return false;
       else
-        return (a.at(0) == '1'); // true if 1
+        return (data.at( 0 ) == '1'); // true if 1
     }
 
     StandardActionManager *q;
@@ -723,7 +720,7 @@ StandardActionManager::StandardActionManager( KActionCollection * actionCollecti
 {
   d->parentWidget = parent;
   d->actionCollection = actionCollection;
-  connect( QApplication::clipboard(), SIGNAL(changed(QClipboard::Mode)), SLOT(clipboardChanged(QClipboard::Mode)) );
+  connect( QApplication::clipboard(), SIGNAL( changed( QClipboard::Mode ) ), SLOT( clipboardChanged( QClipboard::Mode ) ) );
 }
 
 StandardActionManager::~ StandardActionManager()
@@ -734,8 +731,8 @@ StandardActionManager::~ StandardActionManager()
 void StandardActionManager::setCollectionSelectionModel( QItemSelectionModel * selectionModel )
 {
   d->collectionSelectionModel = selectionModel;
-  connect( selectionModel, SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection& )),
-           SLOT(collectionSelectionChanged()) );
+  connect( selectionModel, SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
+           SLOT( collectionSelectionChanged() ) );
 
   d->checkModelsConsistency();
 }
@@ -743,8 +740,8 @@ void StandardActionManager::setCollectionSelectionModel( QItemSelectionModel * s
 void StandardActionManager::setItemSelectionModel( QItemSelectionModel * selectionModel )
 {
   d->itemSelectionModel = selectionModel;
-  connect( selectionModel, SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection& )),
-           SLOT(updateActions()) );
+  connect( selectionModel, SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
+           SLOT( updateActions() ) );
 }
 
 void StandardActionManager::setFavoriteCollectionsModel( FavoriteCollectionsModel *favoritesModel )
@@ -756,8 +753,8 @@ void StandardActionManager::setFavoriteCollectionsModel( FavoriteCollectionsMode
 void StandardActionManager::setFavoriteSelectionModel( QItemSelectionModel *selectionModel )
 {
   d->favoriteSelectionModel = selectionModel;
-  connect( selectionModel, SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection& )),
-           SLOT(favoriteSelectionChanged()) );
+  connect( selectionModel, SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
+           SLOT( favoriteSelectionChanged() ) );
   d->checkModelsConsistency();
 }
 
@@ -785,10 +782,10 @@ KAction* StandardActionManager::createAction( Type type )
   action->setShortcut( actionData[type].shortcut );
 
   if ( actionData[type].slot && !actionData[type].isActionMenu ) {
-    connect( action, SIGNAL(triggered()), actionData[type].slot );
+    connect( action, SIGNAL( triggered() ), actionData[type].slot );
   } else if ( actionData[type].slot ) {
     KActionMenu *actionMenu = qobject_cast<KActionMenu*>( action );
-    connect( actionMenu->menu(), SIGNAL(triggered(QAction*)), actionData[type].slot );
+    connect( actionMenu->menu(), SIGNAL( triggered( QAction* ) ), actionData[type].slot );
   }
 
   d->actionCollection->addAction( QString::fromLatin1(actionData[type].name), action );
