@@ -17,9 +17,10 @@
     02110-1301, USA.
 */
 
+#include "collectionjobtest.h"
+
 #include <sys/types.h>
 
-#include "collectionjobtest.h"
 #include <qtest_akonadi.h>
 #include "test_utils.h"
 #include "testattribute.h"
@@ -44,10 +45,6 @@
 #include "resourceselectjob_p.h"
 #include "collectionfetchscope.h"
 
-#include <QDBusConnection>
-#include <QDBusConnectionInterface>
-
-
 using namespace Akonadi;
 
 QTEST_AKONADIMAIN( CollectionJobTest, NoGUI )
@@ -59,7 +56,7 @@ void CollectionJobTest::initTestCase()
   Control::start();
 
   // switch all resources offline to reduce interference from them
-  foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() )
+  foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() ) //krazy:exclude=foreach
     agent.setIsOnline( false );
 }
 
@@ -512,16 +509,24 @@ void CollectionJobTest::testIllegalModify()
   QVERIFY( !mod->exec() );
 }
 
+void CollectionJobTest::testUtf8CollectionName_data()
+{
+  QTest::addColumn<QString>( "folderName" );
+
+  QTest::newRow( "Umlaut" ) << QString::fromUtf8( "ä" );
+  QTest::newRow( "Garbage" ) << QString::fromUtf8( "đ→³}đþøæſð" );
+}
+
 void CollectionJobTest::testUtf8CollectionName()
 {
-  QString folderName = QString::fromUtf8( "ä" );
+  QFETCH( QString, folderName );
 
   // create collection
   Collection col;
   col.parentCollection().setId( res3ColId );
   col.setName( folderName );
   CollectionCreateJob *create = new CollectionCreateJob( col, this );
-  QVERIFY( create->exec() );
+  AKVERIFYEXEC( create );
   col = create->collection();
   QVERIFY( col.isValid() );
 
@@ -535,7 +540,7 @@ void CollectionJobTest::testUtf8CollectionName()
   // modify collection
   col.setContentMimeTypes( QStringList( "message/rfc822'" ) );
   CollectionModifyJob *modify = new CollectionModifyJob( col, this );
-  QVERIFY( modify->exec() );
+  AKVERIFYEXEC( modify );
 
   // collection statistics
   CollectionStatisticsJob *statistics = new CollectionStatisticsJob( col, this );
@@ -546,7 +551,7 @@ void CollectionJobTest::testUtf8CollectionName()
 
   // delete collection
   CollectionDeleteJob *del = new CollectionDeleteJob( col, this );
-  QVERIFY( del->exec() );
+  AKVERIFYEXEC( del );
 }
 
 void CollectionJobTest::testMultiList()
@@ -597,12 +602,21 @@ void CollectionJobTest::testRidFetch()
   QCOMPARE( col.remoteId(), QString::fromLatin1( "10" ) );
 }
 
+void CollectionJobTest::testRidCreateDelete_data()
+{
+  QTest::addColumn<QString>( "remoteId" );
+  QTest::newRow( "ASCII" ) << QString::fromUtf8( "MY REMOTE ID" );
+  QTest::newRow( "LATIN1" ) << QString::fromUtf8( "MY REMÖTE ID" );
+  QTest::newRow( "UTF8" ) << QString::fromUtf8( "MY REMOTE 検索表" );
+}
+
 void CollectionJobTest::testRidCreateDelete()
 {
+  QFETCH( QString, remoteId );
   Collection collection;
   collection.setName( "rid create" );
   collection.parentCollection().setRemoteId( "8" );
-  collection.setRemoteId( "MY REMOTE ID" );
+  collection.setRemoteId( remoteId );
 
   ResourceSelectJob *resSel = new ResourceSelectJob( "akonadi_knut_resource_2" );
   AKVERIFYEXEC( resSel );
