@@ -3,7 +3,7 @@
 
   Copyright (c) 2001-2003 Cornelius Schumacher <schumacher@kde.org>
   Copyright (c) 2004 Reinhold Kainhofer <reinhold@kainhofer.com>
-  Copyright (c) 2009 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.net>
+  Copyright (c) 2009-2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.net>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 
   @author Cornelius Schumacher \<schumacher@kde.org\>
   @author Reinhold Kainhofer \<reinhold@kainhofer.com\>
-  @author Allen Winter \<allen@kdab.net\>
+  @author Allen Winter \<allen@kdab.com\>
 */
 #ifndef KCAL_INCIDENCEFORMATTER_H
 #define KCAL_INCIDENCEFORMATTER_H
@@ -86,6 +86,24 @@ namespace IncidenceFormatter
                                   bool richText=true,
                                   KDateTime::Spec spec=KDateTime::Spec() );
 
+  /**
+    Create a QString representation of an Incidence in a nice format
+    suitable for using in a tooltip.
+    @param sourceName where the incidence is from (e.g. resource name)
+    @param incidence is a pointer to the Incidence to be formatted.
+    @param date is the QDate for which the toolTip should be computed; used
+    mainly for recurring incidences.
+    @param richText if yes, the QString will be created as RichText.
+    @param spec is an optional time specification which, when specified,
+    will shift the Incidence times to different timezones.
+    @since 4.5
+  */
+  KCAL_EXPORT QString toolTipStr( const QString &sourceName,
+                                  IncidenceBase *incidence,
+                                  const QDate &date=QDate(),
+                                  bool richText=true,
+                                  KDateTime::Spec spec=KDateTime::Spec() );
+
   /**
     Create a QString representation of an Incidence in a nice format
     suitable for using in a tooltip.
@@ -96,9 +114,9 @@ namespace IncidenceFormatter
     @since 4.2
     @deprecated use toolTipStr( Calendar *, IncidenceBase *, bool, KDateTime::Spec)
   */
-  KCAL_EXPORT KDE_DEPRECATED QString toolTipStr( IncidenceBase *incidence,
-                                                 bool richText=true,
-                                                 KDateTime::Spec spec=KDateTime::Spec() );
+  KCAL_EXPORT_DEPRECATED QString toolTipStr( IncidenceBase *incidence,
+                                              bool richText=true,
+                                              KDateTime::Spec spec=KDateTime::Spec() );
 
   /**
     Create a QString representation of an Incidence in a nice format
@@ -107,13 +125,12 @@ namespace IncidenceFormatter
     @param richText if yes, the QString will be created as RichText.
     @deprecated use toolTipStr( Calendar *, IncidenceBase *, bool, KDateTime::Spec)
   */
-  KCAL_EXPORT KDE_DEPRECATED QString toolTipString( IncidenceBase *incidence,
-                                                    bool richText=true );
-
+  KCAL_EXPORT_DEPRECATED QString toolTipString( IncidenceBase *incidence,
+                                                bool richText=true );
   /**
     Create a RichText QString representation of an Incidence in a nice format
     suitable for using in a viewer widget.
-    @parma calendar is a pointer to the Calendar that owns the specified Incidence.
+    @param calendar is a pointer to the Calendar that owns the specified Incidence.
     @param incidence is a pointer to the Incidence to be formatted.
     @param date is the QDate for which the string representation should be computed;
     used mainly for recurring incidences.
@@ -126,6 +143,22 @@ namespace IncidenceFormatter
                                            const QDate &date=QDate(),
                                            KDateTime::Spec spec=KDateTime::Spec() );
 
+  /**
+    Create a RichText QString representation of an Incidence in a nice format
+    suitable for using in a viewer widget.
+    @param sourceName where the incidence is from (e.g. resource name)
+    @param incidence is a pointer to the Incidence to be formatted.
+    @param date is the QDate for which the string representation should be computed;
+    used mainly for recurring incidences.
+    @param spec is an optional time specification which, when specified,
+    will shift the Incidence times to different timezones.
+    @since 4.5
+  */
+  KCAL_EXPORT QString extensiveDisplayStr( const QString &sourceName,
+                                           IncidenceBase *incidence,
+                                           const QDate &date=QDate(),
+                                           KDateTime::Spec spec=KDateTime::Spec() );
+
   /**
     Create a RichText QString representation of an Incidence in a nice format
     suitable for using in a viewer widget.
@@ -135,8 +168,8 @@ namespace IncidenceFormatter
     @since 4.2
     @deprecated use extensiveDisplayStr( Calendar *, IncidenceBase *, KDateTime::Spec )
   */
-  KCAL_EXPORT KDE_DEPRECATED QString extensiveDisplayStr( IncidenceBase *incidence,
-                                                          KDateTime::Spec spec=KDateTime::Spec() );
+  KCAL_EXPORT_DEPRECATED QString extensiveDisplayStr( IncidenceBase *incidence,
+                                                      KDateTime::Spec spec=KDateTime::Spec() );
 
   /**
     Create a RichText QString representation of an Incidence in a nice format
@@ -144,7 +177,7 @@ namespace IncidenceFormatter
     @param incidence is a pointer to the Incidence to be formatted.
     @deprecated use extensiveDisplayStr( Calendar *, IncidenceBase *, KDateTime::Spec )
   */
-  KCAL_EXPORT KDE_DEPRECATED QString extensiveDisplayString( IncidenceBase *incidence );
+  KCAL_EXPORT_DEPRECATED QString extensiveDisplayString( IncidenceBase *incidence );
 
   /**
     Create a QString representation of an Incidence in format suitable for
@@ -157,25 +190,58 @@ namespace IncidenceFormatter
   KCAL_EXPORT QString mailBodyStr( IncidenceBase *incidence,
                                    KDateTime::Spec spec=KDateTime::Spec() );
 
-/**
+  /**
     Create a QString representation of an Incidence in format suitable for
     including inside a mail message.
     @param incidence is a pointer to the Incidence to be formatted.
     @deprecated use mailBodyStr( IncidenceBase *, KDateTime::Spec )
   */
-  KCAL_EXPORT KDE_DEPRECATED QString mailBodyString( IncidenceBase *incidence );
+  KCAL_EXPORT_DEPRECATED QString mailBodyString( IncidenceBase *incidence );
 
   /**
     Deliver an HTML formatted string displaying an invitation.
     Use the time zone from mCalendar.
+
+    @param invitation a QString containing a string representation of a calendar Event
+    which will be intrepreted as an invitation.
+    @param calendar is a pointer to the Calendar that owns the invitation.
+    @param helper is a pointer to an InvitationFormatterHelper.
   */
-  KCAL_EXPORT QString formatICalInvitation( QString invitation, Calendar *mCalendar,
+  KCAL_EXPORT QString formatICalInvitation( QString invitation, Calendar *calendar,
                                             InvitationFormatterHelper *helper );
   /**
-    @since 4.2
+    Deliver an HTML formatted string displaying an invitation.
+    Differs from formatICalInvitation() in that invitation details (summary, location, etc)
+    have HTML formatting cleaned.
+    Use the time zone from calendar.
+
+    @param invitation a QString containing a string representation of a calendar Event
+    which will be intrepreted as an invitation.
+    @param calendar is a pointer to the Calendar that owns the invitation.
+    @param helper is a pointer to an InvitationFormatterHelper.
+    @deprecated use formatICalInvitationNoHtml(const QString &,Calendar *,
+                                               InvitationFormatterHelper *,const QString &) instead.
   */
-  KCAL_EXPORT QString formatICalInvitationNoHtml( QString invitation, Calendar *mCalendar,
-                                                  InvitationFormatterHelper *helper );
+  KCAL_EXPORT QString KDE_DEPRECATED formatICalInvitationNoHtml(
+    QString invitation, Calendar *calendar, InvitationFormatterHelper *helper );
+
+  /**
+    Deliver an HTML formatted string displaying an invitation.
+    Differs from formatICalInvitation() in that invitation details (summary, location, etc)
+    have HTML formatting cleaned.
+    Use the time zone from calendar.
+
+    @param invitation a QString containing a string representation of a calendar Event
+    which will be intrepreted as an invitation.
+    @param calendar is a pointer to the Calendar that owns the invitation.
+    @param helper is a pointer to an InvitationFormatterHelper.
+    @param sender is a QString containing the email address of the person sending the invitation.
+    @since 4.5
+  */
+  KCAL_EXPORT QString formatICalInvitationNoHtml( const QString &invitation,
+                                                  Calendar *calendar,
+                                                  InvitationFormatterHelper *helper,
+                                                  const QString &sender );
 
   /**
     Format a TNEF attachment to an HTML mail
@@ -197,6 +263,17 @@ namespace IncidenceFormatter
   */
   KCAL_EXPORT QString recurrenceString( Incidence *incidence );
 
+  /**
+    Returns a reminder string computed for the specified Incidence.
+    Each item of the returning QStringList corresponds to a string
+    representation of an reminder belonging to this incidence.
+    @param incidence is a pointer to the Incidence.
+    @param shortfmt if false, a short version of each reminder is printed;
+    else a longer version of each reminder is printed.
+    @since 4.5
+  */
+  KCAL_EXPORT QStringList reminderStringList( Incidence *incidence, bool shortfmt = true );
+
   /**
     Build a QString time representation of a KDateTime object.
     @param date The date to be formatted.
@@ -241,6 +318,14 @@ namespace IncidenceFormatter
   */
   KCAL_EXPORT QString resourceString( Calendar *calendar, Incidence *incidence );
 
+  /**
+    Returns a duration string computed for the specified Incidence.
+    Only makes sense for Events and Todos.
+    @param incidence is a pointer to the Incidence.
+    @since 4.5
+  */
+  KCAL_EXPORT QString durationString( Incidence *incidence );
+
   class EventViewerVisitor;
   class ScheduleMessageVisitor;
   class InvitationHeaderVisitor;
