@@ -1,3 +1,24 @@
+/**
+  * This file is part of the KDE project
+  * Copyright (C) 2009 Kevin Ottens <ervin@kde.org>
+  * Copyright (C) 2009 Andras Mantia <amantia@kde.org>
+  *
+  * This library is free software; you can redistribute it and/or
+  * modify it under the terms of the GNU Library General Public
+  * License as published by the Free Software Foundation; either
+  * version 2 of the License, or (at your option) any later version.
+  *
+  * This library is distributed in the hope that it will be useful,
+  * but WITHOUT ANY WARRANTY; without even the implied warranty of
+  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  * Library General Public License for more details.
+  *
+  * You should have received a copy of the GNU Library General Public License
+  * along with this library; see the file COPYING.LIB.  If not, write to
+  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  * Boston, MA 02110-1301, USA.
+  */
+
 #include <kcomponentdata.h>
 #include <kaboutdata.h>
 #include <kdebug.h>
@@ -20,6 +41,7 @@
 #include "kimap/expungejob.h"
 #include "kimap/createjob.h"
 #include "kimap/deletejob.h"
+#include "kimap/namespacejob.h"
 #include "kimap/subscribejob.h"
 #include "kimap/unsubscribejob.h"
 #include "kimap/renamejob.h"
@@ -76,8 +98,9 @@ void listFolders(Session *session, bool includeUnsubscribed = false, const QStri
   int count = list->mailBoxes().size();
   for (int i=0; i<count; ++i) {
     MailBoxDescriptor descriptor = list->mailBoxes()[i];
-    if (descriptor.name.endsWith(nameFilter))
+    if (descriptor.name.endsWith(nameFilter)) {
       kDebug() << descriptor.separator << descriptor.name;
+    }
   }
 
 }
@@ -253,7 +276,9 @@ void testAppendAndStore(Session *session)
 
   QList<QByteArray> resultingFlags = store->resultingFlags()[1];
   qSort(resultingFlags);
-  if (expectedFlags!=resultingFlags) kDebug() << resultingFlags;
+  if (expectedFlags!=resultingFlags) {
+    kDebug() << resultingFlags;
+  }
   Q_ASSERT(expectedFlags==resultingFlags);
 
 
@@ -381,19 +406,19 @@ int main( int argc, char **argv )
 
   QApplication app(argc, argv);
   Session session(server, port);
-  UiProxy *proxy = new UiProxy();
+  UiProxy::Ptr proxy( new UiProxy() );
   session.setUiProxy(proxy);
 
   kDebug() << "Logging in...";
   LoginJob *login = new LoginJob(&session);
-  login->setEncryptionMode(LoginJob::TlsV1);
-  login->setAuthenticationMode(LoginJob::Plain);
+  //login->setEncryptionMode(LoginJob::TlsV1);
+  //login->setAuthenticationMode(LoginJob::Plain);
   login->setUserName(user);
   login->setPassword(password);
   login->exec();
   qDebug();
 
-  if (login->encryptionMode() == LoginJob::Unencrypted)
+  /*if (login->encryptionMode() == LoginJob::Unencrypted)
   {
     kDebug() << "Encrypted login not possible, try to log in without encryption";
     login = new LoginJob(&session);
@@ -404,7 +429,9 @@ int main( int argc, char **argv )
     Q_ASSERT(session.state()==Session::Authenticated);
     qDebug();
 
-  }
+  }*/
+
+  kDebug() << "Server greeting:" << session.serverGreeting();
 
   kDebug() << "Asking for capabilities:";
   CapabilitiesJob *capabilities = new CapabilitiesJob(&session);
@@ -414,6 +441,30 @@ int main( int argc, char **argv )
   kDebug() << capabilities->capabilities();
   qDebug();
 
+  kDebug() << "Asking for namespaces:";
+  NamespaceJob *namespaces = new NamespaceJob(&session);
+  namespaces->exec();
+  Q_ASSERT_X(namespaces->error()==0, "CapabilitiesJob", namespaces->errorString().toLocal8Bit());
+  Q_ASSERT(session.state()==Session::Authenticated);
+
+  kDebug() << "Contains empty namespace:" << namespaces->containsEmptyNamespace();
+
+  kDebug() << "Personal:";
+  foreach ( MailBoxDescriptor ns, namespaces->personalNamespaces() ) {
+    kDebug() << ns.separator << ns.name;
+  }
+
+  kDebug() << "User:    ";
+  foreach ( MailBoxDescriptor ns, namespaces->userNamespaces() ) {
+    kDebug() << ns.separator << ns.name;
+  }
+
+  kDebug() << "Shared:  ";
+  foreach ( MailBoxDescriptor ns, namespaces->sharedNamespaces() ) {
+    kDebug() << ns.separator << ns.name;
+  }
+  qDebug();
+
   kDebug() << "Listing mailboxes:";
   listFolders(&session);
   Q_ASSERT(session.state()==Session::Authenticated);
